# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17


"""

 Contém classes para transformar time series in ARFF 
Problema:
Sabendo dados de tempo=0 até t. Estimar Preco(t+h) pela técnica A, onde h é o horizonte de tempo.

A serie historica total é de 0 até N

Hipoteses simplificadoras:
Tempo Discretizado 
Preço-Real-Final discretizado por Horizonte

Para cada horizonte h define-se Max(h) e numClasses (numero impar>2, para sempre incluir N, H* e L*)

step=2*Max(h)/numClasses
Variacao=(Preco(t+h)/Preco(t)-1)

Limite==Max
indexClasse={L((numclasses-1)/2)...L3,L2,L1,N,H1,H2,H3...H((numclasses-1)/2)}
do 
	Se Variacao<Limite {
	  classe=indexClasse
	  break;
	}
	Se Limite>Max(h) {
	  classe=ultimaClasse
	  break;
	}
	else {
	   indexClasse++; 
	  Limite=Limite+step
	}
while(true);



Preco-real=
Preco-Real= truncarIntervalo[-Max(h)*10,Max(h)*10, Arredondar{(Preco(t+h)/Preco(t)-1)*1000}]
assim Preco-Real será um valor inteiro, (se Max(h)=100 ) com intervalo de variacao de +100% a -100% com 
precisão de 0.1, isto é pode-se prever




Para um dado advisor apenas alguns valores calculados a partir de um certa janela passada sao signficativos.

Digamos n valores das series S1, S2. Seja S1(t-n), ...s1(t) os ultimos dados disponiveis. Dado S1(t+1) seria o primeiro
valor futuro, isto é nao disponivel no momento...

Assim, pode-se reduzir as time series para o arff

S1(n)...s1(1);S2(n)...S2(1);Preço-Real-Final-Horizonte=Preco(t+h)

Onde t =[0,N], o periodo total 

Assim seriam geradas N-n-h instancias para o ARFF definido por um Advisor e um Horizonte


private boolean createArff(String fileName,StringBuffer tab,
			String discretization,int n) {
		try {
			PrintWriter fout = new PrintWriter (new OutputStreamWriter 
					(new FileOutputStream (fileName), "UTF-8"));
			fout.println("@relation technical_"+fileName+".symbolic");

			for (int i = 1; i <= n; i++) 
				fout.println("@attribute open"+i+" numeric");	
			
			for (int i = 1; i <= n; i++) 
				fout.println("@attribute close"+i+" numeric");
			for (int i = 1; i <= n; i++)
				fout.println("@attribute max"+i+"  numeric");
			for (int i = 1; i <= n; i++)
				fout.println("@attribute min"+i+"  numeric");
			for (int i = 1; i <= n; i++)
				fout.println("@attribute avg"+i+"  numeric");
			for (int i = 1; i <= n; i++)
				fout.println("@attribute volume"+i+"  numeric");
			
			fout.println("@attribute numericTarget numeric");
			fout.println("@attribute target {"+discretization+"}");
			fout.println("@data");
			fout.println(tab.toString());
			fout.flush();
			fout.close();
			fout=null;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}

"""
import numpy as np
import pandas as pd


# X is an array of arrays with values 
def ts2Dataset(X,timeFrame):
    ds=np.array()
    for line in range(len(X)-timeFrame):
        for x in X:
            for i in range(timeFrame):
                ds=ds.append(x[line+i])

    n_fields=len(X)
    size=len(ds)
    ds.reshape(size/(n_fields*timeFrame),n_fields*timeFrame)
    return ds
# From bars to dataset
def bars2Dataset(bars,target,timeFrame,horizon=1):
	ds=pd.DataFrame()
	lines=len(bars)
	for time in range(timeFrame):
		for s in bars.keys():
			aux=bars[s][time:lines-horizon-timeFrame+time]
			aux=aux.reset_index(drop=True)
			#del aux['index']
			#print('aux type=',type(aux))
			ds[s+str(time)]=aux
	#print(bars[target][timeFrame+horizon:].shift(-timeFrame),' timeF=',timeFrame,' h=',horizon  )
	aux=bars[target][timeFrame+horizon-1:]
	aux=aux.reset_index(drop=True)
	#del aux['index']
	ds['target']=aux
	return ds

def fromDs2NpArrayAllBut(ds,fieldList):
	all=ds.keys()
	for f in fieldList:
		all=all.drop(f)
	fieldList=[]
	for f in all:
		fieldList.append(f)
	return fromDs2NpArray(ds,fieldList)

# se.ai_utils.get_X( dataframe,features list, time frame ) -> returns a np array
# From bars to dataset
def get_X(df,attr_list,timeFrame,horizon):
	ds=pd.DataFrame()
	lines=len(df)
	for time in range(timeFrame):
		for s in df.keys():
			if s in attr_list:
				aux=df[s][time:lines-timeFrame-horizon+time]
				aux=aux.reset_index(drop=True)
				#del aux['index']
				#print('aux type=',type(aux))
				ds[s+str(time)]=aux
	#print(bars[target][timeFrame+horizon:].shift(-timeFrame),' timeF=',timeFrame,' h=',horizon  )
	#aux=aux.reset_index(drop=True)
	#del aux['index']
	return np.array(ds)


# se.ai_utils.get_X( dataframe,features list, time frame ) -> returns a np array
# From bars to dataset
def get_Y(df,target,timeFrame,horizon):
	Y=pd.DataFrame()
	lines=len(df)
	aux=df[target][timeFrame+horizon:]
	aux=aux.reset_index(drop=True)
	#del aux['index']
	Y['target']=aux

	return np.array(Y)

def get_XY(df,feature_list,target,timeFrame,horizon):
	return get_X(df,feature_list,timeFrame,horizon),get_Y(df,target,timeFrame,horizon)

def fromDs2NpArray(ds,fieldList=[]):
	nfields=len(fieldList)
	#print('nfields=',nfields)
	if nfields==0:
		return None
	elif nfields==1:
		return np.array(ds[fieldList[0]])
	else:
		a=np.array(ds[fieldList[0]])
	
	for i in range(1,nfields):
		a=np.column_stack((a,np.array(ds[fieldList[i]])))
	return a



def discTarget(discretizer,target):
	x=np.array(target)
	x=x.reshape(-1,1)
	dx=discretizer.fit_transform(x) 
	return dx
	
############### temp
#def bars2Dataset(bars,target,timeFrame,horizon=1):
#	ds=pd.DataFrame()
#	new_col = pd.Series([],dtype='float64') 
#	lines=len(bars)
#	for time in range(timeFrame):
#		for s in bars.keys():
#			for i in range(time,lines-horizon-timeFrame+time):
#				#ds[s+str(time)]=bars[s][time:lines-horizon-timeFrame+time].shift(-time)
#				new_col[i]=bars[s][i]
#			ds[s+str(time)]=new_col
#			new_col = pd.Series([])
#	#print(bars[target][timeFrame+horizon:].shift(-timeFrame),' timeF=',timeFrame,' h=',horizon  )
#	new_col = pd.Series([])
#	for i in range(timeFrame+horizon,lines):
#		new_col[i]=bars[target][i]
	#ds['target']=bars[target][timeFrame+horizon:].shift(-time)
#	ds['target']=new_col
#	return ds
##############
def y(y,timeFrame):
    return y[-timeFrame:]


"""
  	for (int line = 0; line < tab.getNumberOfRows()-n-numberOfDays; line++) {
		  	//uma linha 
			  //open
		  	  for (int i = 0; i < n; i++) {
				str.append(tab.get(0,line+i)+",");
			  }
		  	 //close
		  	 for (int i = 0; i < n; i++) {
					str.append(tab.get(1,line+i)+",");
				  }
		  	 //high
		  	 for (int i = 0; i < n; i++) {
					str.append(tab.get(2,line+i)+",");
				  }
		  	//low
		  	 for (int i = 0; i < n; i++) {
					str.append(tab.get(3,line+i)+",");
				  }
		  	//avg
		  	 for (int i = 0; i < n; i++) {
					str.append(tab.get(4,line+i)+",");
				  }
		    //volume
		  	 for (int i = 0; i < n; i++) {
					str.append(tab.get(5,line+i)+",");
			 }
		  	 
            """




# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17

import pandas as pd 
import numpy as np 
from math import sqrt
import statistics 
"""
    calcGeoAvgReturn(returns::Array{Float64} [,n::Int] ) 
 returns the geometric average return of the series of the n first returns returns. If n is not informed the whole array is used 
"""
def calcGeoAvgReturn(returns,n=None):
   ret=1
   if n==None:
      n=len(returns)
   for  i in range(n): 
      ret*=(1+returns[i])
   
   return ret**(1.0/n)-1

 

 
"""
    calcTotalReturn(returns::Array{Float64}) 
   returns the Total return of a series of returns
   If the size is provided it considers just 'size' more recent (more to the right) returns
   left - 0 - (size-1) - right
"""
def calcTotalReturn(returns,size=None):
   ret=1
   s=len(returns)
   if size==None:
      size=0
   elif size<s:
      size=s-size
   else: 
      size=0 # it gets in maximum the return of the total number of elements
   for  i in range(s-1,size-1,-1): 
      ret*=(1+returns[i])
   return ret-1

def changedSignal(returns):
   s=len(returns)
   if s==None or s==1:
      return False
   for i in range(s-1):
      if returns[i]*returns[i+1]<0:
         return True
   return False


"""
    calcAvgReturn(returns::Array{Float64}) 
    returns the arithmetic average return of the series of returns
"""
def calcAvgReturn(returns):
   sum=float(0)
   s=len(returns)
   for  i in range(s): 
      sum+=returns[i]
   return sum/s


"""
    calcAnnualReturn(returns::Array{Float64}, numberOfDays)  
    returns the equivalent annual return for the given serie of returns. The numberOfDays informs the number of working days in the serie, and it
         can have more or less than a year. One year is assumed to have 252 [working] days.
"""
def calcAnnualReturn(returns, numberOfDays):
   gReturn=calcTotalReturn(returns)
   return (1+gReturn)**(252.0/numberOfDays)-1



"""
    calcAnnualSR(returns::Array{Float64}, riskfree, numberOfDays)
    returns the equivalent annual sharpe ratio (SR) for the given serie of returns and risk free rate. The numberOfDays informs the number of working days in the serie, and it
         can have more or less than a year. One year is assumed to have 252 [working] days.
"""
def calcAnnualSR(returns, riskfree, numberOfDays):
   # implemented according paper: Andrew W Lo},The Statistics of Sharpe Ratios, journal = {Financial Analysts Journal}, 2003

   return sqrt(252)*calcSR(returns,riskfree)


"""
    calcStdDev(x::Array{Float64}) 
    return the standard deviation of a sample
"""
def calcStdDev(x):
   return statistics.stdev(x)


"""
    calcSR(returns::Array{Float64}, riskfree)
    returns the Sharpe ratio (SR) for the given serie of returns and risk free rate. 
"""
def calcSR(returns, riskfree):
   avg=calcAvgReturn(returns)
   sigma=calcStdDev(returns)
   if sigma!=0:
       return (avg-riskfree)/sigma
   return -1



"""
    calculates a serie of return given a serie of prices as argument
    return[i]=price[i]/price[i-1]-1
    the serie of returns has length equal to the price serie lenth minus 1.
"""
def calcReturns(serie):
    x=[]
    for i in range(len(serie)-1): # calculates the serie of returns
        x.append(serie[i+1]/serie[i]-1)
    return x


"""
   gives the standard deviation of returns given a serie of prices
"""
def calcStdDevFromPrice(x):
    returns=calcReturns(x)
    return calcStdDev(returns)

"""
   gives the average returns given a serie of prices
"""
def calcAvgReturnFromPrice(x):
    returns=calcReturns(x)
    return calcAvgReturn(returns)




# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17

"""
Operations Module - Disponibiliza funções para facilitar a criação, execução e avaliação de backtests
"""

from mt5se.mt5se import date
import mt5se as se
from datetime import datetime
from datetime import timedelta
import pandas as pd 
import time
import os.path

"""
    Returns the time to the given time in the same day in seconds
"""
def secondsToTime(endHour=18, endMin=0):
    refTime=datetime.now()
    endTime=datetime.now()
    endTime=endTime.replace(hour=endHour, minute=endMin)
    d=endTime-refTime
    return d.total_seconds()

"""
    Returns the datetime expected to end today's session. For default, 17:00 (hour=17 minute=0) 
"""
def sessionEnd(hourSessionEnd=17,minSessionEnd=0):
    endTime=datetime.now()
    endTime=endTime.replace(hour=hourSessionEnd,minute=minSessionEnd,second=0)
    return endTime


"""
 Returns a specification to operations session. Parameters:
    assets,
    capital,
    endTime,
    mem,
    timeframe=se.DAILY,
    file='operation_file',
    verbose=False,
    delay=1,
    waitForOpen=False

"""
def set(assets,capital,endTime,mem,timeframe=se.DAILY,file='operation_file',verbose=False,delay=1,waitForOpen=False):
    ops=dict()  #backtest setup
    if type(waitForOpen)==bool:
        ops['waitForOpen']=waitForOpen
    else:
        print('waitForOpen should be bool')
    if type(verbose)==bool:
        ops['verbose']=verbose
    else:
        print('verbose should be bool')
        return None
    if type(delay)==float or type(delay)==int:
        ops['delay']=delay
    else:
        print('delay should be float')
        return None
    
    if type(mem)==int:
        ops['mem']=mem
    else:
        print('mem should be int')
        return None

    if type(endTime)==datetime:
        ops['end']=endTime
    else:
        print('endTime should be datetime')
        return None
    if timeframe==se.DAILY or timeframe==se.INTRADAY:
        ops['type']=timeframe
    else:
        print('type should be daily or intraday')
        return None
    if type(file)==str:
        ops['file']=file
    else:
        print('file should be str')
        return None
    if type(assets)==list:
        ops['assets']=assets
    else:
        print('assets should be list')
        return None
    if type(capital)==float or type(capital)==int:
        ops['capital']=float(capital)
    else:
        print('capital should be float')
        return None
    return ops

"""
Returns True if the given operation setup is Ok
"""
def checkOps(ops):
    try:
        if type(ops['waitForOpen'])!=bool:
            print('waitForOpen should be bool')
            return False
        if type(ops['verbose'])!=bool:
            print('verbose should be bool')
            return False
        if type( ops['mem'])!=int:
            print('mem should be int')
            return False
        if type( ops['delay'])!=int and type( ops['delay'])!=float:
            print('delay should be int or float. (seconds of delay, between to calls to trade)')
            return False
        if type(ops['start'] )!=datetime:
            print('start should be datetime')
            return False
        if type(ops['end'])!=datetime:
            print('end should be datetime')
            return False
        if ops['type']!=se.DAILY and ops['type']!=se.INTRADAY:
            print('type should be daily or intraday')
            return False
        if type(ops['file'])!=str:
            print('file should be str')
            return False
        if type(ops['assets'])!=list:
            print('assets should be list')
            return False
        if type(ops['capital'])!=float and type(ops['capital'])!=int:
            print('capital should be float')
            return False
        return True
    except:
        print("An exception occurred")
        return False



## assume-se que todos os ativos tem o mesmo numero de barras do ativo indice zero assets[0] no periodo de backtest
sim_dates=[]
balanceHist=[]
equityHist=[]
datesHist=[]
ordersHist=[]
averagePrices=dict()


"""
Returns the current time in a given operation setup
"""
def getCurrTime(ops):
    assets=ops['assets']
    bars=se.get_bars(assets[0],1,timeFrame=se.INTRADAY)
    return bars['time'][0]
    
def startOps(ops): 
    global sim_dates
    assets=ops['assets']
    dbars=dict()
    
    sim_dates.append(getCurrTime(ops))
    mem=ops['mem']
    for asset in assets:
        averagePrices[asset]=0.0
        dbar=se.get_bars(asset,mem,timeFrame=se.INTRADAY)
        if not dbar is None and not dbar.empty:
            dbars[asset]=dbar
        else:
            print("Error asset ",asset, " without information!!!")
    balanceHist.append(ops['capital'])
    equityHist.append(ops['capital'])
    datesHist.append(sim_dates[0])
    ordersHist.append(' ')
    return dbars

def getDeltaOrder(req):
    vol=req['volume']

    if se.isSellOrder(req):
        vol=-vol    
    #elif req['type']==mt5.ORDER_TYPE_BUY_LIMIT or req['type']==mt5.ORDER_TYPE_BUY:
    #    return False
    return vol

""" 
 Returns the Equity in the default currency of the stock
"""
def get_equity():
    return se.account_info().equity


def executeOrders(orders,ops,dbars):
    assets=ops['assets']
    total_in_shares=0.0
    sim_dates.append(getCurrTime(ops))
    executedOrders=[]
    txt=''
    balance=se.get_balance() 
    equity=get_equity()
    for asset in assets:
        shares=se.get_shares(asset)
        order=getOrder(orders,asset)
        # send order
        if order!=None:
            if se.checkOrder(order) and se.sendOrder(order): 
                    print('order sent to se')
                    result=se.getLastOrderResult()
                    executedOrders.append(result)
            else:
                    print('Error  : ',se.getLastError())
            txt=txt+' '+asset+', '+ str(shares)+', '+str(getDeltaOrder(order))+';'
        else:
            txt=txt+' '+asset+','+ str(shares)+', 0;'
            continue
    total_in_shares=se.get_position_value()
    if ops['verbose']==True:
        msg=str(len(orders))+' order(s) in time('+str(sim_dates[-1])+' equity={:,.2f} balance={:,.2f} '+txt
        print(msg.format(equity,balance))
    else:
        msg=str(len(orders))+' order(s) in time('+str(sim_dates[-1])+' equity={:,.2f} balance={:,.2f}. Use verbose=True for more information'
        print(msg.format(equity,balance))
    equityHist.append(equity)  # equity in operations 
    balanceHist.append(balance)
    datesHist.append(sim_dates[-1])
    prices=se.get_last_prices(assets)
    ordersHist.append(orders_to_txt(assets,orders,prices))
    return executedOrders

  
    
def orders_to_txt(assets,orders,prices):
    assets.sort()
    txt=''
    for asset in assets:
        order=getOrder(orders,asset)
        if order is not None:
            if se.isSellOrder(order):
                sinal='-'
            else:
                sinal='+'
            volume=order['volume']
        else:  # sem ordem para o ativo
            sinal=' '
            volume=0
        txt=txt+asset+'/'+sinal+str(volume)+'/'+str(prices[asset])+'/ '
    return txt

def txt_to_orders(txt):
    orders=[]
    lst=txt.split('/')
    if len(lst)%3!=1 or len(lst)<3:
        return None
    i=0
    while i<len(lst)-3:
        order=0
        symbol=lst[i].strip()
        shares=float(lst[i+1])
        price=float(lst[i+2])
        if shares>0:
            order=se.buyOrder(symbol,int(abs(shares)),price)
        elif shares<0:
            order=se.sellOrder(symbol,int(abs(shares)),price)
        else:
            order=None        
        i=i+3
        if order is not None:
            orders.append(order)
    return orders



def getOrder(orders,asset):
    for order in orders:
        if order['symbol']==asset:
            return order
    return None


def getCurrBars(ops,dbars):
    assets=ops['assets']
    #dbars=dict()
    for asset in assets:
        dbar=dbars[asset]
        #pega nova barra    
        aux=se.get_bars(asset,1,timeFrame=se.INTRADAY) # pega uma barra!
        if not aux is None and not aux.empty:
            dbar=dbar.iloc[1:,] #remove barra mais antiga
            #adiciona nova barra
            dbar=dbar.append(aux)
            dbar.index=range(len(dbar))# corrige indices
            dbars[asset]=dbar
       
    return dbars 

def getLastTime(ops):
    assets=ops['assets']
    bars=se.get_bars(assets[0],1,timeFrame=se.INTRADAY)
    return bars['time'][0]
   

def endedOps(ops):
    assets=ops['assets']
    if not se.is_market_open(assets[0]):
        print('Market is NOT open at the moment!!')
        return True
    
    if ops['verbose']:
        print('Ended?? time =',getCurrTime(ops), ' of ',len(sim_dates))
    if ops['end']==None:
        return True
    elif ops['end']<getLastTime(ops):
        return 
    else:
        return False




"""
    Start the execution of the given trader according to the operation setup given
"""
def run(trader,ops):
    se.mt5se.inbacktest=False

    if trader==None: # or type(trader)!=se.Trader:
        print("Error! Trader should be an object of class mt5se.Trader or its subclass")
        return False
    dbars=startOps(ops)
    assets=ops['assets']
    trader.setup(dbars)
    if 'delay' in ops.keys():
        delay=ops['delay']
    else:
        delay=0
    if ops['verbose']:
        print("Starting Operation at date/time=",sim_dates[0]," len=",len(sim_dates))
    if ops['waitForOpen']:
        while not se.is_market_open(assets[0]):
            print('Market is NOT open! we will wait until it is open...')
            time.sleep(1)
    while not endedOps(ops):
        orders=trader.trade(dbars)
        ex_order_list=executeOrders(orders,ops,dbars)
        trader.orders_result(ex_order_list)
        dbars=getCurrBars(ops,dbars)
        saveTick(ops)
        time.sleep(delay)
    print('End of operation saving equity file in ',ops['file'])
    trader.ending(dbars)
    df=saveEquityFile(ops)
    return df


def saveTick(ops):
    """
    print('csv format, columns: <DATE>		<BALANCE>	<EQUITY>	<DEPOSIT LOAD>')
<DATE>	            <BALANCE>	<EQUITY>	<DEPOSIT LOAD>
2019.07.01 00:00	100000.00	100000.00	0.0000
2019.07.01 12:00	99980.00	99999.00	0.0000
2019.07.01 12:59	99980.00	100002.00	0.1847
2019.07.01 12:59	99980.00	99980.00	0.0000
2019.07.02 14:59	99960.00	99960.00	0.0000
2019.07.03 13:00	99940.00	99959.00	0.0000
2019.07.03 13:59	99940.00	99940.00	0.0000
2019.07.08 15:59	99920.00	99936.00	0.0000
2019.07.08 16:59	99920.00	99978.00	0.1965
2019.07.10 10:00	99920.00	99920.00	0.0000
2019.07.10 10:59	99900.00	99937.00	0.1988
Formato gerado pelo metatrader,
ao fazer backtest com o Strategy Tester, clicar na tab 'Graphs' e botao direto 'Export to CSV (text file)'
    """
    #print('write report....')
    if len(equityHist)!=len(balanceHist) or len(balanceHist)!=len(datesHist) or len(datesHist)!=len(ordersHist):
        print("Erro!! Diferentes tamanhos de historia, de equity, balance e dates")
        print('bH=',len(balanceHist),' dH=',len(datesHist),' orderHist=',len(ordersHist))
        return False
    if len(equityHist)<=0:
        return False
    df=pd.DataFrame()
    df['date']=[]
    df['balance']=[]
    df['equity']=[]
    df['load']=[]
    df['orders']=[]
    # Salva apenas o ultimo tick
    #for i in range(len(equityHist)):
    
    i=len(equityHist)-1
    idx=len(df)
    df.loc[idx]=[datesHist[i],balanceHist[i],equityHist[i],0.0,ordersHist[i]]

    if os.path.isfile(ops['file']+'.csv'):
        df.to_csv(ops['file']+'.csv',mode='a',header=False) # file already exists, so it appends
    else:
        df.to_csv(ops['file']+'.csv')
    return df 





def saveEquityFile(ops):
    """
    print('csv format, columns: <DATE>		<BALANCE>	<EQUITY>	<DEPOSIT LOAD>')
<DATE>	            <BALANCE>	<EQUITY>	<DEPOSIT LOAD>
2019.07.01 00:00	100000.00	100000.00	0.0000
2019.07.01 12:00	99980.00	99999.00	0.0000
2019.07.01 12:59	99980.00	100002.00	0.1847
2019.07.01 12:59	99980.00	99980.00	0.0000
2019.07.02 14:59	99960.00	99960.00	0.0000
2019.07.03 13:00	99940.00	99959.00	0.0000
2019.07.03 13:59	99940.00	99940.00	0.0000
2019.07.08 15:59	99920.00	99936.00	0.0000
2019.07.08 16:59	99920.00	99978.00	0.1965
2019.07.10 10:00	99920.00	99920.00	0.0000
2019.07.10 10:59	99900.00	99937.00	0.1988
Formato gerado pelo metatrader,
ao fazer backtest com o Strategy Tester, clicar na tab 'Graphs' e botao direto 'Export to CSV (text file)'
    """
    #print('write report....')
    if len(equityHist)!=len(balanceHist) or len(balanceHist)!=len(datesHist):
        print("Erro!! Diferentes tamanhos de historia, de equity, balance e dates")
        return False
    df=pd.DataFrame()
    df['date']=[]
    df['balance']=[]
    df['equity']=[]
    df['load']=[]
    df['orders']=[]

    for i in range(len(equityHist)):
        df.loc[i]=[datesHist[i],balanceHist[i],equityHist[i],0.0,ordersHist[i]]

    if not os.path.isfile(ops['file']+'.csv'): # salva apenas se nao existir!!
       # df.to_csv(ops['file']+'.csv',mode='a',header=False) # file already exists, so it appends
        #else:
        df.to_csv(ops['file']+'.csv')
    return df 





# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17

import numpy.random as rand
import mt5se as se
import time
import pandas as pd
import numpy as np

class RandomTrader(se.Trader):
    def __init__(self):
        pass

    def setup(self,dbars):
        print('just getting started!')

    def trade(self,dbars):
        orders=[] 
        assets=list(dbars.keys())
        for asset in assets:
            if rand.randint(2)==1:     
                order=se.buyOrder(asset,100)
            else:
            	order=se.sellOrder(asset,100)
            orders.append(order)
        return orders
    
    def ending(self,dbars):
        print('Ending stuff')

 

class MonoAssetTrader(se.Trader):
    def trade(self,dbars):
        assets=dbars.keys()
        asset=list(assets)[0]
        orders=[]
        bars=dbars[asset]
        curr_shares=se.get_shares(asset)
        # number of shares that you can buy
        price=se.get_last(bars)
        free_shares=se.get_affor_shares(asset,price)
        rsi=se.tech.rsi(bars)
        if rsi>=70:   
            order=se.buyOrder(asset,free_shares)
        else:
            order=se.sellOrder(asset,curr_shares)
        if rsi>=70 and free_shares>0: 
            order=se.buyOrder(asset,free_shares)
        elif  rsi<70 and curr_shares>0:
            order=se.sellOrder(asset,curr_shares)
        if order!=None:
                orders.append(order)
        return orders   



class MultiAssetTrader(se.Trader):
    def trade(self,dbars):
        assets=dbars.keys()
        orders=[]
        for asset in assets:
            bars=dbars[asset]
            curr_shares=se.get_shares(asset)
            money=se.get_balance()/len(assets) # divide o saldo em dinheiro igualmente entre os ativos
            # number of shares that you can buy of asset 
            price=se.get_last(bars)
            free_shares=se.get_affor_shares(asset,price,money)
            rsi=se.tech.rsi(bars)
            if rsi>=70 and free_shares>0: 
                order=se.buyOrder(asset,free_shares)
            elif  rsi<70 and curr_shares>0:
                order=se.sellOrder(asset,curr_shares)
            else:
                order=None
            if order!=None:
                orders.append(order)
        return orders   


from sklearn import tree
from sklearn.preprocessing import KBinsDiscretizer
 
class SimpleAITrader(se.Trader):

    def setup(self,dbars):
        assets=list(dbars.keys())
        if len(assets)!=1:
            print('Error, this trader is supposed to deal with just one asset')
            return None
        bars=dbars[assets[0]]
        # remove irrelevant info
        if 'time' in bars:
            del bars['time']
        timeFrame=10 # it takes into account the last 10 bars
        horizon=1 # it project the closing price for next bar
        target='close' # name of the target column
        ds=se.ai_utils.bars2Dataset(bars,target,timeFrame,horizon)
        
        X=se.ai_utils.fromDs2NpArrayAllBut(ds,['target'])
        discretizer = KBinsDiscretizer(n_bins=3, encode='ordinal', strategy='uniform') 
        # creates the discrete target
        ds['target']=se.ai_utils.discTarget(discretizer,ds['target'])
        Y=se.ai_utils.fromDs2NpArray(ds,['target'])

        clf = tree.DecisionTreeClassifier()

        clf = clf.fit(X, Y)
        self.clf=clf

    def trade(self,dbars):
            assets=dbars.keys()
            orders=[]
            timeFrame=10 # it takes into account the last 10 bars
            horizon=1 # it project the closing price for next bar
            target='close' # name of the target column
            money=se.get_balance()/len(assets) # shares the balance equally among the assets
            for asset in assets:
                bars=dbars[asset]
                curr_shares=se.get_shares(asset)
                price=se.get_last(bars)
                free_shares=se.get_affor_shares(asset,price,money)
                # get new information (bars), transform it in X
                bars=dbars[asset]
                #remove irrelevant info
                if 'time' in bars:
                    del bars['time']
                # convert from bars to dataset
                ds=se.ai_utils.bars2Dataset(bars,target,timeFrame,horizon)
                # Get X fields
                X=se.ai_utils.fromDs2NpArrayAllBut(ds,['target'])

                # predict the result, using the latest info
                p=self.clf.predict([X[-1]])
                if p==2:
                    #buy it
                    order=se.buyOrder(asset,free_shares)
                elif p==0:
                    #sell it
                    order=se.sellOrder(asset,curr_shares)
                else:
                    order=None
                if order!=None:
                    orders.append(order)
            return orders    



from sklearn.ensemble import RandomForestClassifier
#from sklearn.preprocessing import KBinsDiscretizer

class RandomForestTrader(se.Trader):

    def setup(self,dbars):
        assets=list(dbars.keys())
        if len(assets)!=1:
            print('Error, this trader is supposed to deal with just one asset')
            return None
        bars=dbars[assets[0]]
        # remove irrelevant info
        if 'time' in bars:
            del bars['time']
        timeFrame=10 # it takes into account the last 10 bars
        horizon=1 # it project the closing price for next bar
        target='close' # name of the target column
        ds=se.ai_utils.bars2Dataset(bars,target,timeFrame,horizon)
        
        X=se.ai_utils.fromDs2NpArrayAllBut(ds,['target'])
        discretizer = KBinsDiscretizer(n_bins=3, encode='ordinal', strategy='uniform') 
        # creates the discrete target
        ds['target']=se.ai_utils.discTarget(discretizer,ds['target'])
        Y=se.ai_utils.fromDs2NpArray(ds,['target'])

        #clf = tree.DecisionTreeClassifier()
        clf = RandomForestClassifier(n_estimators=10)
        clf = clf.fit(X, Y)
        self.clf=clf

    def trade(self,dbars):
            assets=dbars.keys()
            orders=[]
            timeFrame=10 # it takes into account the last 10 bars
            horizon=1 # it project the closing price for next bar
            target='close' # name of the target column
            money=se.get_balance()/len(assets) # shares the balance equally among the assets
            for asset in assets:
                bars=dbars[asset]
                curr_shares=se.get_shares(asset)
                price=se.get_last(bars)
                free_shares=se.get_affor_shares(asset,price,money)
                # get new information (bars), transform it in X
                bars=dbars[asset]
                #remove irrelevant info
                if 'time' in bars:
                    del bars['time']
                # convert from bars to dataset
                ds=se.ai_utils.bars2Dataset(bars,target,timeFrame,horizon)
                # Get X fields
                X=se.ai_utils.fromDs2NpArrayAllBut(ds,['target'])

                # predict the result, using the latest info
                p=self.clf.predict([X[-1]])
                if p==2:
                    #buy it
                    order=se.buyOrder(asset,free_shares)
                elif p==0:
                    #sell it
                    order=se.sellOrder(asset,curr_shares)
                else:
                    order=None
                if order!=None:
                    orders.append(order)
            return orders    




"""
  Monoaset Monoanalyst Trader, it may be used to evaluate one analyst in respect of one asset
"""
class AnalystTrader(se.Trader):
    def __init__(self,analyst):
        pass
        self.analyst=analyst
        self.dates=[]
        self.current=[]
        self.predicted=[]
        self.actual=[]   

    def setup(self,dbars):
        self.analyst.setup(dbars)
        
    
    def trade(self,ops,dbars):
        orders=[] 
        assets=ops['assets']
        asset=assets[0]
        if len(assets)!=1:
            print('Error!! Analyst trader should manage one and just one asset!!!')
            exit()
        mu=self.analyst.analyze(dbars)# Analysts returns a dictionary with the target prices for each asset
        bars=dbars[asset]
        p=se.get_last(bars) # last price  TODO: esta pegando o preco atual ao inves do seguinte a predicao
        #p_1=bars['close'].iloc[-2]
       #old  actual_return=(p/p_1)**252-1 #actual price #  old (annualized) return from the last cycle (p[t]/p[t-1])^252-1
        self.dates.append(bars['time'].iloc[-1])
        self.current.append(p)
        self.predicted.append(mu[asset])
        self.actual.append(p)
        return orders #AnalystTrader always return no orders!!

    def saveAnalystFile(self):
            print('save analyst file')
            df=pd.DataFrame()
            df['date']=[]
            df['current']=[]
            df['predicted']=[] # predicted (annualized) return 
            df['actual']=[]  # actual (annualized) return from the last cycle (p[t]/p[t-1])^252-1
            #df['price']=[]   #asset price

            for i in range(len(self.dates)):
                df.loc[i]=[self.dates[i],self.current[i],self.predicted[i],self.actual[i]]
            df.to_csv('analyst_performance.csv') 






# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17


"""
Backtest Module - Disponibiliza funções para facilitar a criação, execução e avaliação de backtests
"""

import mt5se as se
from datetime import datetime
from datetime import timedelta
import pandas as pd 
import numpy as np
import os.path


def set(assets,prestart,start,end,period,capital,file='backtest_file',verbose=False):

    bts=dict()  #backtest setup
    if type(verbose)==bool:
        bts['verbose']=verbose
    else:
        print('verbose should be bool')
        return None
    if type(prestart)==datetime:
        bts['prestart']=prestart
    else:
        print('prestart should be datetime')
        return None
    if type(start)==datetime:
        bts['start']=start
    else:
        print('start should be datetime')
        return None
    if type(end)==datetime:
        bts['end']=end
    else:
        print('end should be datetime')
        return None
    if period==se.DAILY or period==se.INTRADAY or period==se.H1:
        bts['type']=period
    else:
        print('type should be daily or intraday or H1')
        return None
    if type(file)==str:
        bts['file']=file
    else:
        print('file should be str')
        return None
    if type(assets)==list:
        bts['assets']=assets
    else:
        print('assets should be list')
        return None
    if type(capital)==float or type(capital)==int:
        bts['capital']=float(capital)
    else:
        print('capital should be float')
        return None
    return bts

def get_shares(bts,asset):
    return bts['shares_'+asset]

def get_balance(bts):
    return bts['capital']



## assume-se que todos os ativos tem o mesmo numero de barras do ativo indice zero assets[0] no periodo de backtest
sim_dates=[]

def startBckt(bts): 
    global sim_dates
    assets=bts['assets']
    dbars=dict()
    for asset in assets:
        dbars[asset]=se.get_bars(asset,bts['prestart'],bts['start'],bts['type'])
        bts['shares_'+asset]=0.0
    bars=se.get_bars(assets[0],bts['start'],bts['end'],bts['type'])
    
    sim_dates=bars['time']
   
    bts['curr']=0 # guarda a data simulada corrente como indice de sim_dates
    
    #balanceHist.append(bts['capital'])
    #equityHist.append(bts['capital'])
    #datesHist.append(sim_dates[bts['curr']])
    return dbars

def endedBckt(bts):
    if bts['verbose']:
        print('Ended?? time =', bts['curr'], ' of ',len(sim_dates))
    if bts['curr']==None or bts['end']==None:
        return True
    elif bts['curr']<len(sim_dates):
        return False
    else:
        return True


balanceHist=[]
equityHist=[]
datesHist=[]
ordersHist=[]

def checkOrder(req,bts,bars):
    if req==None:
        return False
    money=bts['capital']
    asset=req['symbol']
    volume=req['volume']
    price=se.get_last(bars)
    sell=se.isSellOrder(req)

    if sell:
        if bts['shares_'+asset]>=volume:
            return True
        else:
            return False
    else:
        if money>=volume*price : # checa se não ficaria negativo com a execução
            return True
        else:
            se.setLastError('Trade would make the balance negative! Therefore, it does not check!')
        return False


def compute_order(order,volume,price):
    lastOrderResult=dict()
    lastOrderResult['symbol']=order['symbol'] 
    lastOrderResult['isSellOrder']=se.isSellOrder(order)
    lastOrderResult['shares']=volume
    lastOrderResult['price']=price
    return lastOrderResult





def computeOrders(orders,bts,dbars):
    assets=bts['assets']
    total_in_shares=0.0
    executedOrdersList=[]
    if orders==None:
        equityHist.append(equityHist[-1])
        balanceHist.append(balanceHist[-1])
        datesHist.append(sim_dates[bts['curr']])
        for asset in assets:
            bar=dbars[asset]
            price=se.get_last(bar)
            total_in_shares=total_in_shares+bts['shares_'+asset]*price # counts the value in asset with no order
        if bts['verbose']:
            print( 'No orders in time(',bts['curr'],') = ',sim_dates[bts['curr']],' capital=',bts['capital'], 'total in shares=',total_in_shares)
        return True
    
    if bts['verbose']:
        print('List of ',len(orders),'orders in time(',bts['curr'],') :')
    for asset in assets:
        bar=dbars[asset]
        if bar is None:
            print('Error accesing bar to compute order')
            return False
        price=se.get_last(bar)
        order=getOrder(orders,asset)
        if order==None: # if no order for that asset, go to the next
            total_in_shares=total_in_shares+bts['shares_'+asset]*price # counts the value in asset with no order
            continue
        volume=order['volume']
        if se.isSellOrder(order):
            bts['shares_'+asset]=bts['shares_'+asset]-volume
            bts['capital']=bts['capital']+volume*price
            if bts['verbose']:
                print("Order for selling ",volume,"shares of asset=",asset, " at price=",price)
        else:
            bts['shares_'+asset]=bts['shares_'+asset]+volume
            bts['capital']=bts['capital']-volume*price
            if bts['verbose']:
                print("Order for buying ",volume,"shares of asset=",asset, " at price=",price)
        ord_result=compute_order(order,volume,price)
        executedOrdersList.append(ord_result)
        total_in_shares=total_in_shares+float(bts['shares_'+asset])*price # counts the value in asset with order
    if bts['verbose']:
        print( len(orders),' order(s) in time(',bts['curr'],') = ',sim_dates[bts['curr']],' capital=',bts['capital'], 'total in shares=',total_in_shares, 'equity=',bts['capital']+total_in_shares)
    equityHist.append(bts['capital']+total_in_shares)
    balanceHist.append(bts['capital'])
    datesHist.append(sim_dates[bts['curr']])
    #detalhamento das ordens
    prices=se.get_last_prices(assets)
    ordersHist.append(se.operations.orders_to_txt(assets,orders,prices))
    return executedOrdersList
    

def getOrder(orders,asset):
    for order in orders:
        if order['symbol']==asset:
            return order
    return None


def getCurrBars(bts,dbars):
    assets=bts['assets']
    #dbars=dict()
    for asset in assets:
        dbar=dbars[asset]
        #pega nova barra    
        aux=se.get_bars(asset,sim_dates[bts['curr']],1,bts['type']) # pega uma barra! daily or intraday
        if not aux is None and not aux.empty:
            dbar=dbar.iloc[1:,] #remove barra mais antiga
            #adiciona nova barra
            #dbar=dbar.append(aux)
            pd.concat([dbar, aux], ignore_index=True)
            dbar.index=range(len(dbar))# corrige indices
            dbars[asset]=dbar
       
    return dbars 

def checkBTS(bts):
    try:
        if type(bts['verbose'])!=bool:
            print('verbose should be bool')
            return False
        if type( bts['prestart'])!=datetime:
            print('prestart should be datetime')
            return False
        if type(bts['start'] )!=datetime:
            print('start should be datetime')
            return False
        if type(bts['end'])!=datetime:
            print('end should be datetime')
            return False
        if bts['type']!=se.DAILY and bts['type']!=se.INTRADAY and bts['type']!=se.H1:
            print('type should be daily or intraday or H1')
            return False
        if type(bts['file'])!=str:
            print('file should be str')
            return False
        if type(bts['assets'])!=list:
            print('assets should be list')
            return False
        if type(bts['capital'])!=float and type(bts['capital'])!=int:
            print('capital should be float')
            return False
        return True
    except:
        print("An exception occurred")
        return False

def run(trader,bts):
    se.mt5se.inbacktest=True
    se.mt5se.bts=bts
    balanceHist.clear()
    equityHist.clear()
    datesHist.clear()
    if trader==None: # or type(trader)!=se.Trader:
        print("Error! Trader should be an object of class mt5se.Trader or its subclass")
        return False
    if not checkBTS(bts):
        print("The Backtest setup (bts) is not valid!")
        return False
    dbars=startBckt(bts)
    trader.setup(dbars)
    bts['curr']=0
    if bts['verbose']:
        print("Starting at simulated date=",sim_dates[0]," len=",len(sim_dates))
    while not endedBckt(bts):
        #orders=trader.getNewInfo(dbars)
        orders=trader.trade(dbars)
        dbars=getCurrBars(bts,dbars)
        ex_orders_list=computeOrders(orders,bts,dbars)
        trader.orders_result(ex_orders_list)
        if bts['verbose']:
            print("Advancing simulated date from ",bts['curr']," = ",sim_dates[bts['curr']])
        bts['curr']=bts['curr']+1 # advances simulated time
    print('End of backtest with ',bts['curr'],' bars,  saving equity file in ',bts['file'])
    trader.ending(dbars)
    df=saveEquityFile(bts)
    se.mt5se.inbacktest=False
    return df


def saveEquityFile(bts):
    """
    print('csv format, columns: <DATE>		<BALANCE>	<EQUITY>	<DEPOSIT LOAD>')
<DATE>	            <BALANCE>	<EQUITY>	<DEPOSIT LOAD> <orders>
2019.07.01 00:00	100000.00	100000.00	0.0000
2019.07.01 12:00	99980.00	99999.00	0.0000
2019.07.01 12:59	99980.00	100002.00	0.1847
2019.07.01 12:59	99980.00	99980.00	0.0000
2019.07.02 14:59	99960.00	99960.00	0.0000
2019.07.03 13:00	99940.00	99959.00	0.0000
2019.07.03 13:59	99940.00	99940.00	0.0000
2019.07.08 15:59	99920.00	99936.00	0.0000
2019.07.08 16:59	99920.00	99978.00	0.1965
2019.07.10 10:00	99920.00	99920.00	0.0000
2019.07.10 10:59	99900.00	99937.00	0.1988
Formato gerado pelo metatrader,
ao fazer backtest com o Strategy Tester, clicar na tab 'Graphs' e botao direto 'Export to CSV (text file)'
    """
    #print('write report....')
    if len(equityHist)!=len(balanceHist) or len(balanceHist)!=len(datesHist):
        print("Erro!! Diferentes tamanhos de historia, de equity, balance e dates")
        return False
    df=pd.DataFrame()
    df['date']=[]
    df['balance']=[]
    df['equity']=[]
    df['load']=[]
    df['orders']=[]

    for i in range(len(equityHist)):
        df.loc[i]=[datesHist[i],balanceHist[i],equityHist[i],0.0,ordersHist[i]]

    if os.path.isfile(bts['file']+'.csv'):
        df.to_csv(bts['file']+'.csv',mode='a',header=False) # file already exists, so it appends
    else:
        df.to_csv(bts['file']+'.csv') 
    return df 


def evaluate(df):
   #rreturns=__calcReturns(df['equity'])
   """ print('---rreturns------')
   print(rreturns)
   for r in rreturns:
       print(r)
   print('---rreturns------') """
   #if df==None:
   #    print('Error!! df should be a DataFrame with a equity column')
   evaluateEquitySerie(df['equity'])



#using Distributions
#using CSV
#using StringEncodings
from math import sqrt
from scipy.stats import norm
from scipy.stats import kurtosis
from scipy.stats import skew

import pandas as pd 
import numpy as np 
import statistics 




"""
   return the probability of the performance greater than the given threshold (annual return)
"""
def ProbReturnGreaterThanThreshold(returns,threshold):
   numberOfDays=len(returns)
   if numberOfDays<30:
      print("In order to perform STSE evaluation, you should have at least 30 daily data points, but you got only ",numberOfDays)
      return False
   prob=__estimateProb(returns,(threshold+1)**(1/252)-1)
   return 1-prob
"""
   https://www.google.com/search?q=how+to+obtain+a+distribution+from+another+distribution&oq=how+to+obtain+a+distribution+from+another+distribution&aqs=chrome..69i57.71322j0j4&sourceid=chrome&ie=UTF-8#kpvalbx=_q39vX-3TA9ix5OUPl9Sg4AI30

   y=g(x)= (1+x)^252-1

   Fy(y)=P(Y<=y)
   Y<=y
   g(x)=(1+x)^252-1<=y
   x<=(y+1)^(1/252)-1=expr
   3.Fy(y)=P(Y<=y)=P(x<expr)
   Logo,
   Probabilidade de retorno maior que y=1-Fy(y)=1-P(X<(y+1)^(1/252)-1)
"""

def __estimateProb(returns,limit):
   numberOfDays=len(returns)
   if numberOfDays<30:
      print("In order to perform evaluation, you should have at least 30 daily data points, but you got only ",numberOfDays)
      return False
   if returns==None or len(returns)==0:
      return 0
   smaller=0
   for i in returns:
      if i<=limit:
         smaller=smaller+1
   return smaller/numberOfDays



"""
    calcGeoAvgReturn(returns::Array{Float64}) 
    returns the geometric average of the given serie of returns. 
"""
def calcGeoAvgReturn(returns):
   ret=1
   s=len(returns)
   for  i in range(s):
      ret*=(1+returns[i])
   return ret**(1.0/s)-1

"""
    calcStdDev(x::Array{Float64}) 
    return the standard deviation of a sample
"""
def calcStdDev(x):
   return statistics.stdev(x)


"""
    calculates a serie of return given a serie of prices as argument
    return[i]=price[i]/price[i-1]-1
    the serie of returns has length equal to the price serie lenth minus 1.
"""
def calcReturnsFromPrice(serie):
   x=[]
   if type(serie)==pd.Series:
      for i,valor in serie.iteritems(): # calculates the serie of returns
         x.append(valor)
      y=[]
      for i in range(len(x)-1): # calculates the serie of returns
        y.append(x[i+1]/x[i]-1)
      return y   
   else:
    for i in range(len(serie)-1): # calculates the serie of returns
        x.append(serie[i+1]/serie[i]-1)
    return x


def __calcReturns(serie):
    x=[]
    #print('calcRetursn')
    for i in range(len(serie)-1): # calculates the serie of returns
        x.append(serie[i+1]/serie[i]-1)
        #print(x[i])
    return x


"""
    evaluateEquitySerie(serie,threshold=0.5,riskFree=0.0)
    evaluates a trader performance given its serie of historical equity value 
"""
def evaluateEquitySerie(serie,threshold=0.5,riskFree=0.0):
    if serie is None:
        print("serie should be a list of observed market values of the portfolio, given daily")
        return None
    serie=__calcReturns(serie)
    numberOfDays=len(serie)
    if numberOfDays<30:
        print("In order to perform evaluation, you should have at least 30 data points, but you got only ",numberOfDays)
        return False
    print("\n -----------------------   Backtest Report  ------------------------------- \n")
    print("Total Return (%)={:.2f} in {} bars ".format(calcTotalReturn(serie)*100,numberOfDays))
    print("Average Bar Return (%)={:.2f}  ".format(np.average(serie)*100))
    #print("Annualized Return (%)={:.2f}".format(calcAnnualReturn(serie,numberOfDays)*100))
    print("Std Deviation of returns (%) ={:.4f}".format(calcStdDev(serie)*100))
    #print("Sharpe Ratio={:.4f} ".format(calcSharpeRatio(serie,riskFree)))
    #print("Annualized Sharpe Ratio={:.4f} ".format(calcAnnualSharpeRatio(serie,riskFree,numberOfDays ))),
    """ l1=0
   p1=ProbReturnGreaterThanThreshold(serie,l1)
   l2=0.1
   p2=ProbReturnGreaterThanThreshold(serie,l2)
   l3=0.2
   p3=ProbReturnGreaterThanThreshold(serie,l3)
   
   print("Probability that Annual Return is greater than ({:.1f}%) ={:.2f}%".format(100*l1, 100*p1))
   print("Probability that Annual Return is greater than ({:.1f}%) ={:.2f}%".format(100*l2, 100*p2))
   print("Probability that Annual Return is greater than ({:.1f}%) ={:.2f}%".format(100*l3, 100*p3))"""

    print("\n ----------------------        End of Report     -------------------------------- \n")



"""
    processFile(fileName,numberOfDays)
    process the  "tick-returns CSV file" pointed by fileName and provide several information about the strategy performance. The numberOfDays informs the number of working days in the serie, and it
    can have more or less than a year. One year is assumed to have 252 [working] days. 
"""
def evaluateFile(fileName,threshold=0.5,riskFree=0.0):
  
  # assetSR=calcSharpeRatio(areturns,0)
   cv=pd.read_csv(fileName)

   #rreturns=__calcReturns(cv['equity'])
   #evaluateEquitySerie expectes the equity serie
   evaluateEquitySerie(cv['equity'],threshold,riskFree)


#returns the Total return of a series of returns given of the n first returns
def calcTotalReturn(returns):
   ret=1
   s=len(returns)
   for  i in range(s):
      ret*=(1+returns[i])
   return ret-1


#returns the arithmetic average return of the series of returns given of the n first returns
def calcAvgReturn(returns): 
   sum=float(0)
   s=len(returns)
   for  i in range(s):
      sum+=returns[i]
   return sum/s


def calcAnnualReturn(returns, numberOfDays):
   gReturn=calcTotalReturn(returns)
   return (1+gReturn)**(252.0/numberOfDays)-1



def calcAnnualSharpeRatio(returns, riskfree, numberOfDays):
   # annRet=calcAnnualReturn(returns,numberOfDays)
   # sigma=calcDesvPad(returns) # we suppose sigma stable
   #if(sigma==0) 
   #   print("ERROR! standard deviation equals to zero. In calc Annual SR")
   #   return (annRet-riskfree)
   #end
   #return (annRet-riskfree)/sigma
   # implemented according to paper Andrew W Lo},The Statistics of Sharpe Ratios, journal = {Financial Analysts Journal}, 2003

   return sqrt(252)*calcSharpeRatio(returns,riskfree)


def calcSharpeRatioFromPrice(prices, riskfree): 
   returns=calcReturnsFromPrice(prices)
   return calcSharpeRatio(returns,riskfree)

def calcSharpeRatio(returns, riskfree): 
   avg=calcAvgReturn(returns)
   sigma=calcStdDev(returns)
   if sigma!=0:
       return (avg-riskfree)/sigma
   print("Error!! standard deviation of returns is not suposed to be zero, but it is!!")
   return -1





# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17


# mt5se main module
import MetaTrader5 as mt5
import pandas as pd 
import numpy as np 
import mt5se.backtest as backtest
import random
#from math import *
from datetime import datetime
from datetime import timedelta
# importamos o módulo pytz para trabalhar com o fuso horário
import pytz
from pytz import timezone



#sptz=pytz.timezone('Brazil/East')
etctz=pytz.timezone('etc/utc') # os tempos sao armazenados na timezone ETC (Greenwich sem horario de verao)

path=None # Metatrader program file path
datapath=None # Metatrader path to data folder
commonDatapath=None # Metatrader common data path
company=None  #broker name
platform=None  # digital plataform (M)
connected=False
inbacktest=False
bts=None
DAILY=mt5.TIMEFRAME_D1 # daily bars
INTRADAY=mt5.TIMEFRAME_M1 # 1 minute bars
H1=mt5.TIMEFRAME_H1 # 1 hour bars
# comprehensive list of time frames
TIMEFRAME_M1=mt5.TIMEFRAME_M1
TIMEFRAME_M2=mt5.TIMEFRAME_M2
TIMEFRAME_M3=mt5.TIMEFRAME_M3
TIMEFRAME_M4=mt5.TIMEFRAME_M4
TIMEFRAME_M5=mt5.TIMEFRAME_M5
TIMEFRAME_M6=mt5.TIMEFRAME_M6
TIMEFRAME_M10=mt5.TIMEFRAME_M10
TIMEFRAME_M12=mt5.TIMEFRAME_M12
TIMEFRAME_M15=mt5.TIMEFRAME_M15
TIMEFRAME_M20=mt5.TIMEFRAME_M20
TIMEFRAME_M30=mt5.TIMEFRAME_M30
TIMEFRAME_H1=mt5.TIMEFRAME_H1
TIMEFRAME_H2=mt5.TIMEFRAME_H2
TIMEFRAME_H3=mt5.TIMEFRAME_H3
TIMEFRAME_H4=mt5.TIMEFRAME_H4
TIMEFRAME_H6=mt5.TIMEFRAME_H6
TIMEFRAME_H8=mt5.TIMEFRAME_H8
TIMEFRAME_H12=mt5.TIMEFRAME_H12
TIMEFRAME_D1=mt5.TIMEFRAME_D1
TIMEFRAME_W1=mt5.TIMEFRAME_W1
TIMEFRAME_MN1=mt5.TIMEFRAME_MN1




def connect(account=None,passw=None,mt5path=None):
    """
   Connects to the specificied account, password and specifiec (by path) metatrader installation 
        if parameters are not specified it connects to last used account and password
"""
    if account is None and passw is None:
        if mt5path is None:
            res= mt5.initialize()
        else:
            res= mt5.initialize(mt5path)
    else:
        account=int(account)
        if mt5path is None:
            res= mt5.initialize(login=account, password=passw)
        else:
            res= mt5.initialize(mt5path,login=account, password=passw)
    global ac,path,datapath,commonDatapath,company,platform,connected
    if res!=True:
        if account is None:
            print('Error trying to connect to last account!!', ' Error code:',mt5.last_error())
        else:
            print('Error trying to connect to account: ',account, ' Error code:',mt5.last_error())
        return False
    info=mt5.account_info()
    if info.margin_so_mode !=mt5.ACCOUNT_MARGIN_MODE_RETAIL_NETTING and info.margin_so_mode !=mt5.ACCOUNT_MARGIN_MODE_EXCHANGE:
        # ACCOUNT_MARGIN_MODE_RETAIL_NETTING is usually used for simulated accounts
        # ACCOUNT_MARGIN_MODE_EXCHANGE is usually used for real accounts
        print("It is NOT netting, but the stock exchange should be netting trade mode!! Error!!")  # B3 is also Netting!!
        return False
    #elif info.margin_so_mode ==mt5.ACCOUNT_MARGIN_MODE_RETAIL_HEDGING:
    #    print("It is hedding, not netting")
    #else:
    #    print("It is something elese!!")
    #if info.margin_so_mode ==mt5.ACCOUNT_MARGIN_MODE_RETAIL_NETTING:
    #    print("It is netting, not hedding")  # se is Netting!!
    #elif info.margin_so_mode ==mt5.ACCOUNT_MARGIN_MODE_RETAIL_HEDGING:
    #    print("It is hedding, not netting")
    #else:
    #    print("It is something elese!!")
    if res:
        ac=mt5.terminal_info()
        path=ac.path
        datapath=ac.data_path
        commonDatapath=ac.commondata_path
        company=ac.company
        platform=ac.name
        connected=True
    return res

def terminal_info():
    """
    Returns info regarding the terminal in a dicitionary 
        For instance:
        'community_account': False,
        'community_connection': False,
        'connected': True, 
        'dlls_allowed': True, 
        'trade_allowed': False,
        'tradeapi_disabled': False,
        'email_enabled': False,
        'ftp_enabled': False,
        'notifications_enabled': False,
        'mqid': False,
        'build': 2875,
        'maxbars': 100000,
        'codepage': 0,
        'ping_last': 18717, 
        'community_balance': 0.0, 
        'retransmission': 0.0, 
        'company': 'MetaQuotes Software Corp.', 
        'name': 'MetaTrader 5', 
        'language': 'English', 
        'path': 'C:\\Program Files\\MetaTrader 5',
        'data_path': 'C:\\Users\\..\\AppData\\Roaming\\MetaQuotes\\Terminal\\D...', 
        'commondata_path': 'C:\\Users\\...\\AppData\\Roaming\\MetaQuotes\\Terminal\\Common

"""
    if not connected:
        print("In order to use this function, you must be connected to the . Use function connect()")
        return
    term_info = mt5.terminal_info()
    return term_info

def account_info(): 
    """
    Returns info about the account as a dicitionary
    For instance:
  login=25115284
  trade_mode=0
  leverage=100
  limit_orders=200
  margin_so_mode=0
  trade_allowed=True
  trade_expert=True
  margin_mode=2
  currency_digits=2
  fifo_close=False
  balance=9...
  credit=0.0
  profit=41.82
  equity=9.
  margin=9..
  margin_free=9..
  margin_level=1..
  margin_so_call=50.0
  margin_so_so=30.0
  margin_initial=0.0
  margin_maintenance=0.0
  assets=0.0
  liabilities=0.0
  commission_blocked=0.0
  name=MetaQuotes Dev Demo
  server=MetaQuotes-Demo
  currency=USD
  company=MetaQuotes Software Corp.

"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    account_info = mt5.account_info()
    return account_info


def get_shares(symbolId):
    """
    Returns the current number of assets of the given symbol.
        It will be negative if there is a short position! and zero if there is no position
"""
    global inbacktest
    global bts
    if inbacktest:
        #print('Esta em backtest. bts=', bts)
        return backtest.get_shares(bts,symbolId)
    #else:
      #  print('NAO esta em backtest')
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    pos= mt5.positions_get(symbol=symbolId)
    if pos is not None and pos!=():
        d=pos[0]._asdict() 
        if d['type']==1:
            return -1*d['volume']
        else:
            return d['volume']
    else:
        return 0




def is_market_open(asset='B3SA3'):
    """
  It returns if the market is open or not for new orders.
     Note that markets can close in different times for different assets, therefore
     you need to inform the target asset. The default target assets is B3 stock (it will work only in B3 stock exchange)
     For other exchanges inform a valid asset ticker (For instance, GOOG should work for Nasdaq).
     It there is no tick for 60 seconds, the market is considered closed!
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
   # si=mt5.symbol_info(asset)
   # if siis not None:
  #      if si.trade_mode==mt5.SYMBOL_TRADE_MODE_FULL: # it does not work in XP/B3 (always True)
  #          return True
  #      else:
  #          return False
  #  return False
    mt5.symbol_select(asset) # it makes sure that the symbol is present in Market Watch View
    t_secs=mt5.symbol_info_tick(asset).time # time in seconds
    now_dt=datetime.now(etctz)+timedelta(hours=-3)
    last_tick_dt=datetime.fromtimestamp(t_secs,etctz)
    #print(last_tick_dt)
    #print(now_dt)
    if now_dt>last_tick_dt+timedelta(seconds=60):
        return False
    else: 
        return True



def today(offset=None):
    """
Returns the today datetime (hours=minutes=seconds=0)
 You can define an offset in days.
    today(1) returns tomorrow
    today(-1) returns yesterday
"""
    dt=datetime.now()
    dt=datetime(dt.year,dt.month,dt.day,0,0,0)
    if offset is None:
        return dt
    if type(offset)!=int:
        print('Offset should a int with the number of dates. Returning today date without offset')
        return dt
    else:
        dt=dt+timedelta(days=offset)
        return dt


def now(dayOffest=None,hourOffset=None,minOffset=None):
    """
    Returns the current datetime 
 You can define an offset in days,hours and minutes
    today(dayOffset=1) returns tomorrow
    today(dayOffset=-1) returns yesterday
    today(dayOffset=-1,hourOffset=-2,minOffset=-30) returns yesterday two hours and 30 minutes 
"""
    dt=datetime.now()
    if dayOffest is not None:
        dt=dt+timedelta(days=dayOffest)
    if hourOffset is not None:
        dt=dt+timedelta(hours=hourOffset)
    if minOffset is not None:
        dt=dt+timedelta(minutes=minOffset) 
    return dt

"""
  It returns if the market is still open but just for closing orders.
     Note that markets can close in different times for different assets, therefore
     you need to inform the target asset. The default target assets is B3 stock
"""
#def isMarketClosing(asset='B3SA3'): # it does not work in XP/B3 (always false)
#    si=mt5.symbol_info(asset)
#    if si is not None:
 #       if si.trade_mode==mt5.SYMBOL_TRADE_MODE_CLOSEONLY:
  #          return True
  #      else:
  #          return False
  #  return False



def get_volume_step(assetId):
    """
  Returns the volume step for an asset
"""
    step=mt5.symbol_info(assetId).volume_step
    return step
    

def get_affor_shares(assetId,price,money=None,volumeStep=None):
    """
    Returns the max volume of shares thay you can buy, with your balance
        it also observes the volume step (a.k.a minimum number of shares you can trade)
"""
    global inbacktest
    global bts
    if inbacktest:
        #print('Esta em backtest. bts=')#, bts)
        if money is None:
            money=bts['capital']
        return pget_affor_shares(assetId,price,money,volumeStep)
    #else:
      #  print('NAO esta em backtest')
    return pget_affor_shares(assetId,price,money,volumeStep)

def pget_affor_shares(assetId,price,money=None,volumeStep=None):
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    if money is None:
        money=mt5.account_info().margin_free
    if money <=0:
        return 0.0
        
    if price is None:
        close=mt5.symbol_info_tick(assetId).last
    else:
        close=price
    if volumeStep is None:
        step=get_volume_step(assetId)
    else:
        step=volumeStep    
    free=0
    while free*close<money:
        free=free+step
        #print('free=',free, ' close=',close,' money=',money  )
    return free-step


def get_balance():
    """ 
 Returns the Account balance (free resource) in the default currency of the stock
        It is equivalent to free margin in MT5 jargon
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    global inbacktest
    global bts
    if inbacktest:
        #print('Esta em backtest. bts=')#, bts)
        return backtest.get_balance(bts)
    #else:
      #  print('NAO esta em backtest')
    return mt5.account_info().margin_free





def get_position_value(symbolId=None):
    """
 Returns the current value in portfolio of a given symbol, or total value of alls symbol, if None is given
"""
    pos=get_positions(symbolId)
    if len(pos)>0:
        return sum(pos['volume']*pos['price_current'])
    else:
        return 0.0

def get_positions(symbolId=None): 
    """
 Returns a pandas dataframe with the position of a given symbol or group of symbols.
    If the parameter is None, it returns all current positions in the account
#Example:
    se.get_positions('AAPL')
  symbol  type   volume  price_open  price_current
0  EMBR3     1  47000.0       14.04          16.02

    pos['price_current']  #current price of the asset
 #print("get position")
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    if symbolId is None:
        positions=mt5.positions_get()
    else:
        positions=mt5.positions_get(symbol=symbolId)
    if len(positions)>0:
        df=pd.DataFrame(list(positions),columns=positions[0]._asdict().keys())
        df=df[['symbol','type','volume', 'price_open', 'price_current']]
        return df
    else:
        return pd.DataFrame()



def buyOrder(symbolId,volume,price=None,sl=None,tp=None): # Buying !!
    """
Creates and returns a buy order with the given symbolId, volume, price [optional], stop loss[optional], take profit [optional]
    An order is a dictionary with at least the following fields:
    "action": class of order 
    "symbol":  symbol id
    "volume": float 
    "type":  ORDER_TYPE_SELL,
    "deviation": 
    "magic": 
    "comment":
    "type_time":
    "type_filling": 
    "price":
    See also: checkOrder(ord), isSellOrder(ord), isBuyOrder(ord)
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    symbol_info = mt5.symbol_info(symbolId)
   #print("symbol=",symbolId," info=",symbol_info)
    if symbol_info is None:
        setLastError(symbolId + " not found, can not create buy order")
        return None
 
# se o símbolo não estiver disponível no MarketWatch, adicionamo-lo
    if not symbol_info.visible:
            #print(symbolId, "is not visible, trying to switch on")
            if not mt5.symbol_select(symbolId,True):
                setLastError("symbol_select({}}) failed! symbol=" +symbolId)
                return None   
    #point = mt5.symbol_info(symbolId).point
    deviation = 20
    
    request = {
        "action": mt5.TRADE_ACTION_DEAL,
        "symbol": symbolId,
        "volume": float(volume),
        "type": mt5.ORDER_TYPE_BUY,

        "deviation": deviation,
        "magic": random.randrange(100,100000),
        "comment": "order by mt5se",
        "type_time": mt5.ORDER_TIME_GTC,
        "type_filling": mt5.ORDER_FILLING_RETURN,
        }
    if price is None:  # order a mercado
        request['action']=mt5.TRADE_ACTION_DEAL
        request['type']=mt5.ORDER_TYPE_BUY
        request['price']=mt5.symbol_info_tick(symbolId).ask
    else:  # order limitada
        request['action']=mt5.TRADE_ACTION_PENDING
        request['type']=mt5.ORDER_TYPE_BUY_LIMIT
        request['price']=float(price)
    if sl is not None:
        request["sl"]=sl
    if tp is not None:
            request["tp"]= tp
    

    return request



def sellOrder(symbolId,volume,price=None,sl=None,tp=None): # Selling !!
    """
Creates and returns a sell order with the given symbolId, volume, price [optional], stop loss[optional], take profit [optional].
    An order is a dictionary with at least the following fields:
    "action": class of order 
    "symbol":  symbol id
    "volume": float 
    "type":  ORDER_TYPE_SELL,
    "deviation": 
    "magic": 
    "comment":
    "type_time":
    "type_filling": 
    "price":
    See also: checkOrder(ord), isSellOrder(ord), isBuyOrder(ord)
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    symbol_info = mt5.symbol_info(symbolId)
    #print("symbol=",symbolId," info=",symbol_info)
    if symbol_info is None:
        setLastError(symbolId + " not found, can not create buy order")
        return None
    # se o símbolo não estiver disponível no MarketWatch, adicionamo-lo
    if not symbol_info.visible:
        #print(symbolId, "is not visible, trying to switch on")
        if not mt5.symbol_select(symbolId,True):
            setLastError("symbol_select({}}) failed! symbol=" +symbolId)
            return None   
    point = mt5.symbol_info(symbolId).point
    deviation = 20
    request = {
    "action": mt5.TRADE_ACTION_DEAL,
    "symbol": symbolId,
    "volume": float(volume),
    "type": mt5.ORDER_TYPE_SELL,
    "deviation": deviation,
    "magic": random.randrange(100,100000),
    "comment": "order by mt5se",
    "type_time": mt5.ORDER_TIME_DAY,
    "type_filling": mt5.ORDER_FILLING_FOK,
    }
    if price is None:  # order a mercado
       request['action']=mt5.TRADE_ACTION_DEAL
       request['type']=mt5.ORDER_TYPE_SELL
       request['price']=mt5.symbol_info_tick(symbolId).ask
    else:  # order limitada
       request['action']=mt5.TRADE_ACTION_PENDING
       request['type']=mt5.ORDER_TYPE_SELL_LIMIT
       request['price']=float(price)
    if sl is not None:
       request["sl"]=sl
   
    if tp is not None:
        request["tp"]= tp
    return request


def isSellOrder(req):
    """
    Returns true if it is a sell order, False if it is a buy order.
        It sets a descriptive error message (getLastError / setLastError) if there is an error
"""
    if req is None:
        print("Error! Order is None!!!!")
        return False
    if req['type']==mt5.ORDER_TYPE_SELL_LIMIT or req['type']==mt5.ORDER_TYPE_SELL:
        return True
    elif req['type']==mt5.ORDER_TYPE_BUY_LIMIT or req['type']==mt5.ORDER_TYPE_BUY:
        return False
    else:
        print("Error! Order is not buy our sell!!!!")
        return False



def isBuyOrder(req):
    """
    Returns true if it is a sell order, False if it is a buy order.
        It sets a descriptive error message (getLastError / setLastError) if there is an error
"""
    if req is None:
        print("Error! Order is None!!!!")
        return False
    if req['type']==mt5.ORDER_TYPE_SELL_LIMIT or req['type']==mt5.ORDER_TYPE_SELL:
        return False
    elif req['type']==mt5.ORDER_TYPE_BUY_LIMIT or req['type']==mt5.ORDER_TYPE_BUY:
        return True
    else:
        print("Error! Order is not buy our sell!!!!")
        return False



def checkOrder(req):
    """
    Returns true if it a valid order
    It returns false in case of Short order or insufficient money to buy
"""
    global inbacktest
    if inbacktest:
        #print('Esta em backtest. bts=')#, bts)
        return backtest.get_balance(bts)
    #else:
      #  print('NAO esta em backtest')
    if req is None:
        return False
    result = mt5.order_check(req)
    #print('result=',result, 'req=',req)
    if result is None: # error
        setLastError(mt5.last_error())
        return False
    d=result._asdict()
    #margin - Margem requerida para a operação de negociação
    # margin_free - Margem livre que sobrará após a execução da operação de negociação
    # balance  - Valor de saldo após a execução da operação de negociação
    #for k in d.keys():
    #    print('{} = {}',k,d[k])
    if isSellOrder(req):
        if req['volume']>get_shares(req['symbol']): #bloqueia ordem a descoberto
            return False
        else:
            return True

    if d['balance']>=0 : # checa se não ficaria negativo com a execução
        return True
    else:
        setLastError('Trade would make the balance negative! Therefore, it does not check!')
        return False



lastErrorText=""

def getLastError():
    """
    Returns a string with the last error's message
"""
    global lastErrorText
    if lastErrorText is None or lastErrorText=="":
        return mt5.last_error()
    else:
        aux=lastErrorText
        lastErrorText=None  
        return aux    
 
def setLastError(error):
    """
    Sets the last error's message. Used together with getLastError
"""  
    global lastErrorText
    lastErrorText=error

lastOrderResult=""
def setLastOrderResult(order,result):
    """"
       set the result of the last order. Used together with getLastOrderResult
            result is a dictionary with four keys:
            'symbol'  - asset id
            'isSellOrder' - True if it is a sell order, false if it is a buy order
            'shares'   - Number of shares of the order
            'price'    -  Executed price for the order
    """
    global lastOrderResult
    lastOrderResult=dict()
    lastOrderResult['symbol']=order['symbol'] 
    lastOrderResult['isSellOrder']=isSellOrder(order)
    lastOrderResult['shares']=result.volume
    lastOrderResult['price']=result.price
    return lastOrderResult


def getLastOrderResult():
    """
    Returns the result of the last succesful sent order
        result is a dictionary with four keys:
            'symbol'  - asset id
            'isSellOrder' - True if it is a sell order, false if it is a buy order
            'shares'   - Number of shares of the order
            'price'    -  Executed price for the order
"""
    global lastOrderResult
    return lastOrderResult




def sendOrder(order):
    """
    Sends the given order to online execution 
        returns True if sucessfull and False otherwise
    Obs: This function should be used only for direct control traders
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    if order is None:
        return False
    # enviamos a solicitação de negociação
    result = mt5.order_send(order)
    if result.retcode != mt5.TRADE_RETCODE_DONE:  # if error
        print("Sent order failed < {} > retcode={}".format(result.comment,result.retcode))
        # solicitamos o resultado na forma de dicionário e exibimos elemento por elemento
        dic=result._asdict()
        setLastError(dic['comment'])
       # for field in dic.keys():
       #     print("   {}={}".format(field,dic[field]))
       #     #se esta for uma estrutura de uma solicitação de negociação, também a exibiremos elemento a elemento
       #     if field=="request":
       #         traderequest_dict=dic[field]._asdict()
       #         for tradereq_filed in traderequest_dict:
       #             print("       traderequest: {}={}".format(tradereq_filed,traderequest_dict[tradereq_filed]))
        return False
    else:
        setLastOrderResult(order,result)  # set the result of last sucessful order 
        return True


def numOrders(): 
    """
 Returns the number of active orders
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    result=mt5.orders_total()
    if result is None:
        setLastError("Error on getting orders total")
        return -1
    else:
        return result


def get_active_orders():  
    """
    Returns a dataframe with all active orders
order fields  description:
    #order_id | symbol | type | price | sl | tp | 
    # type: 0 - Buy (Market Order), 1  - Sell (Market Order)
            2 - Buy (LImited Order), 3 - Sell (Limited Order)
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    orders=mt5.orders_get()
    df=pd.DataFrame()
    if orders is None or len(orders)==0:
        return df
    else:
        dfaux=pd.DataFrame(list(orders),columns=orders[0]._asdict().keys())
        df['order_id']=dfaux['ticket']
        df['symbol']=dfaux['symbol']
        df['price']=dfaux['price_open']
        df['type']=dfaux['type']
        return df


def cancel_order(order_id):  
    """
    Cancels a order specified by its order_id (see get_orders).
        Returns True if the order is sucessfuly cancelled! False otherwise
            If the order does not exist, it also returns False
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    order_id=int(order_id)
    order=mt5.orders_get(ticket=order_id)
    if order is None:
        return False
    remove=dict()
    remove['action']=mt5.TRADE_ACTION_REMOVE
    remove['order']=order_id
    return mt5.order_send(remove)


def getDailYBars(symbol, start,end=None): # sao inclusas barras com  tempo de abertura <= end.
    # definimos o fuso horário como UTC
    #timezone = pytz.timezone("Etc/UTC")
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    if end is None:
        end=datetime.now()
    if type(start).__name__!='datetime':
        if type(start).__name__!='int':
            print('Error, start should be a datetime from package datetime or int')
        else:
            start_day=datetime.now() #- timedelta(days=start)
            rates=mt5.copy_rates_from(symbol,mt5.TIMEFRAME_D1,start_day,start)
             # criamos a partir dos dados obtidos DataFrame
            rates_frame=pd.DataFrame(rates)
            rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
            return rates_frame
    else:
       rates=mt5.copy_rates_range(symbol,mt5.TIMEFRAME_D1,start,end)
       # criamos a partir dos dados obtidos DataFrame
       rates_frame=pd.DataFrame(rates)
       rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
       return rates_frame


def get_returns(bars):
    """
  Returns a serie of returns from given bars using open-close prices
    close[i]/open[i]-1
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    x=[]
    for i in range(len(bars)):
        x.append(bars['close'][i]/bars['open'][i]-1)
    return x


def get_last(bars): # argumento deve ser bars nao vazia, retorna erro se estiver vazia
    """
  Returns the last close price from given bars
"""
    if bars is None:
        return 0.0
    return bars['close'].iloc[-1]


def get_first(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
    """
  Returns the first open price from given bars
"""
    if bars is None:
        return 0
    return bars['open'][0]

def get_max(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
    """
  Returns the max high price from given bars
"""
    if bars is None:
        return 0
    return max(bars['high'])


def get_min(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
    """
  Returns the min low price from given bars
"""
    if bars is None:
        return 0
    return max(bars['low'])


def get_last_time(bars): # argumento deve ser bars nao vazia, retorna erro se estiver vazia
    """
  Returns the last time from given bars
"""
    if bars is None:
        return 0
    return bars['time'].iloc[-1]

def get_first_time(bars):# argumento deve ser bars nao vazia, retorna erro se estiver vazia
    """
  Returns the first (earlier) time from given bars
"""
    if bars is None:
        return 0
    return bars['time'][0]


def read_bars_file(fileName):
    """
 Returns a pandas data frame with the content of the bars file given as parameter.
    Bars files are csv files that can be obtained from MetaTrader with the following nine columns:
     date,time,open,high,low,close,vol, tickvol,spread
"""
    df=pd.read_csv(fileName,delimiter='\t',names=['date','time','open','high','low','close','vol', 'tickvol','spread'],header=0) 
    if df is None or len(df.columns)!=9:
        print("The bars file should be a csv file with nine columns: date,time,open,high,low,close,vol, tickvol,spread")
        return None
    else:
        return df

def get_bars(symbol, start,end=None,timeFrame=DAILY):
    """
 Returns a pandas data frame with bars information. Parameters
    symbol - the asset symbol
    start - the start date or the number of the last x desired bars
    end - the end date [optional] 
    timeFrame - the bars time frame, it may be DAILY (default) (1 day bars) or INTRADAY (1 minute bars)
        There are others possible time frames, for 1 minute (TIMEFRAME_M1), 2 minutes, 1 hour (TIMEFRAME_H1), two hours, 1 week(TIMEFRAME_W1), one month(TIMEFRAME_MN1), etc.
            TIMEFRAME_M1
            TIMEFRAME_M2
            TIMEFRAME_M3
            TIMEFRAME_M4
            TIMEFRAME_M5
            TIMEFRAME_M6
            TIMEFRAME_M10
            TIMEFRAME_M12
            TIMEFRAME_M15
            TIMEFRAME_M20
            TIMEFRAME_M30
            TIMEFRAME_H1
            TIMEFRAME_H2
            TIMEFRAME_H3
            TIMEFRAME_H4
            TIMEFRAME_H6
            TIMEFRAME_H8
            TIMEFRAME_H12
            TIMEFRAME_D1
            TIMEFRAME_W1
            TIMEFRAME_MN1
    For instance,
        se.get_bars('AAPL',10) # returns the last 10 daily bars    
        time   open   high    low  close  tick_volume  spread  real_volume
"""
 # definimos o fuso horário como UTC
    #timezone = pytz.timezone("Etc/UTC")
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    if symbol is None or type(symbol)!=str:
        return None
    else:
        symbol=symbol.upper()
    if timeFrame in [DAILY,H1,INTRADAY,TIMEFRAME_M1,TIMEFRAME_M2,TIMEFRAME_M3,TIMEFRAME_M4	,TIMEFRAME_M5,TIMEFRAME_M6,TIMEFRAME_M10,TIMEFRAME_M12,TIMEFRAME_M15,  \
    TIMEFRAME_M20,TIMEFRAME_M30,TIMEFRAME_H1,TIMEFRAME_H2,TIMEFRAME_H3,TIMEFRAME_H4,TIMEFRAME_H6,TIMEFRAME_H8,TIMEFRAME_H12,TIMEFRAME_D1,TIMEFRAME_W1,TIMEFRAME_MN1]:
        # if it is a valid time frame it just continues
        pass
    else:  # if timeframe is invalid, assumes it is daily
        timeFrame=mt5.TIMEFRAME_D1
    if end is None:
        #if timeFrame!=mt5.TIMEFRAME_M1:
        end=datetime.now()
        #else: # intraday, end=None then end is the same as start
        #    end=start
    if type(start).__name__!='datetime' and type(start).__name__!='Timestamp':
        if type(start).__name__!='int':
            print('Error, start should be a datetime or int, but it is ',type(start).__name__)
            return None
        else:
            start_day=datetime.now() #- timedelta(days=start)
            rates=mt5.copy_rates_from(symbol,timeFrame,start_day,start)
             # criamos a partir dos dados obtidos DataFrame
            rates_frame=pd.DataFrame(rates)
            if len(rates_frame)>0:
                rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
            return rates_frame
    else:
        if type(end).__name__=='int':
            rates=mt5.copy_rates_from(symbol,timeFrame,start,end)
        else:
            rates=mt5.copy_rates_range(symbol,timeFrame,start,end)
       # criamos a partir dos dados obtidos DataFrame
        rates_frame=pd.DataFrame(rates)
        if len(rates_frame)>0:
            rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
        return rates_frame




def get_multi_bars(assets,start,end=None,type=DAILY):
    """
Returns bars for multiple assets. It is similar to get_bars (that deals with just one asset)
    mbars=get_multi_bars(assets,start,end)
    mbars[assets[0]] # bars for the first asset (0)
"""
    dbars=dict()
    for asset in assets:
        dbars[asset]=get_bars(asset,start,end,type)
    return dbars




def get_orders(start,end,allFields=False):
    """
Returns list of orders for a given period of time  (start-end)as pandas DataFrame
    Parameters: start,end (datetime from datetime)
                allFields=False (by default only the main fields are returned)
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    lis2=mt5.history_orders_get(start,end)
    df2=pd.DataFrame(list(lis2),columns=lis2[0]._asdict().keys())
    if not allFields:
        df2.drop(['time_setup_msc','time_done_msc','time_setup_msc','time_expiration','type_time','state','position_by_id','reason','volume_current','price_stoplimit','sl','tp'], axis=1, inplace=True)
    df2['time_setup'] = pd.to_datetime(df2['time_setup'], unit='s')
    df2['time_done'] = pd.to_datetime(df2['time_done'], unit='s')
    #df2['type'] =df2['type'].map(type_order)
    #df2['reason'] =df2['reason'].map(reason)
    return df2



def get_deals(start,end,allFields=False):
    """
Returns list of deals for a given period of time  (start-end)as pandas DataFrame
    deals - include executed orders and orders executed by the broker (margin calls, withdraws, deposits and others)
    Parameters: start,end (datetime from datetime)
                allFields=False (by default only the main fields are returned)
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    lis2=mt5.history_deals_get(start,end)
    df2=pd.DataFrame(list(lis2),columns=lis2[0]._asdict().keys())
    if not allFields:
        df2.drop(['entry','swap','external_id','time_msc','magic','order','position_id'], axis=1, inplace=True) 
    df2['time']=pd.to_datetime(df2['time'], unit='s')
    #df2['type'] =df2['type'].map(type_order)
    #df2['reason'] =df2['reason'].map(reason)
    return df2




# Funcoes auxiliares para type and reason of order and deals
def order_type(x):
    """ 
        Returns a string with type of order 
        0 - ORDER_TYPE_BUY - Market Buy order
        1 - ORDER_TYPE_SELL - Market Buy order
        2 - ORDER_TYPE_BUY_LIMIT - Buy Limit pending order
        3 - ORDER_TYPE_SELL_LIMIT - Sell Limit pending order
        4 - ORDER_TYPE_BUY_STOP - Buy Stop pending order
        5 - ORDER_TYPE_SELL_STOP - Sell Stop pending order
        6 - ORDER_TYPE_BUY_STOP_LIMIT - Upon reaching the order price, a pending Buy Limit order is placed at the StopLimit price
        7 - ORDER_TYPE_SELL_STOP_LIMIT - AUpon reaching the order price, a pending Sell Limit order is placed at the StopLimit price
        8 - ORDER_TYPE_CLOSE_BY - Order to close a position by an opposite one
    """
    if x==0:
        #Market Buy order
        return 'BUY'
    elif x==1:
        #Market Sell order
        return 'SELL'
    elif x==2: 
        #Buy Limit pending order
        return 'LIMTED_BUY'
    elif x==3:
        #Sell Limit pending order
        return 'LIMITED_SELL'
    elif x==4:
        #Buy Stop pending order
        return 'ORDER_TYPE_BUY_STOP'
    elif x==5:
        #Sell Stop pending order
        return 'ORDER_TYPE_SELL_STOP'
    elif x==6:
        #Upon reaching the order price, a pending Buy Limit order is placed at the StopLimit price
        return 'ORDER_TYPE_BUY_STOP_LIMIT'
    elif x==7:
        #Upon reaching the order price, a pending Sell Limit order is placed at the StopLimit price
        return 'ORDER_TYPE_SELL_STOP_LIMIT'
    elif x==8:
        #Order to close a position by an opposite one
        return 'ORDER_TYPE_CLOSE_BY'
    else:
        return 'UNKNOWN'


def order_reason(x):
    """
        Returns a string with the reason of a given order. Possible values:
        0 - ORDER_REASON_CLIENT - The order was placed from a desktop terminal
        1 - ORDER_REASON_MOBILE - The order was placed from a mobile application
        2 - ORDER_REASON_WEB - The order was placed from a web platform
        3 - ORDER_REASON_EXPERT - The order was placed from an MQL5-program, i.e. by an Expert Advisor or a script
        4 - ORDER_REASON_SL - The order was placed as a result of Stop Loss activation
        5 - ORDER_REASON_TP - The order was placed as a result of Take Profit activation
        6 - ORDER_REASON_SO - The order was placed as a result of the Stop Out event
    """
    if x==0:
        # Ordem colocada a partir de um terminal desktop
        return 'ORDER_REASON_CLIENT'
    elif x==1:
        #Ordem colocada a partir de um aplicativo móvel
        return 'ORDER_REASON_MOBILE'
    elif x==2:
        #Ordem colocada a partir da plataforma web
        return 'ORDER_REASON_WEB'
    elif x== 3:
        #Ordem colocada a partir de um programa MQL5, Expert Advisor ou script
        return 'ORDER_REASON_EXPERT'
    elif x== 4:
        #Ordem colocada como resultado da ativação do Stop Loss
        return 'ORDER_REASON_SL'
    elif x== 5:
        #Ordem colocada como resultado da ativação do Take Profit
        return 'ORDER_REASON_TP'
    elif x== 6:
        #Ordem colocada como resultado do evento Stop Out
        return 'ORDER_REASON_SO'
    else:
        return 'UNKNOWN'



def deal_type(x):
    """
        Returns a string with type of a deal. Possible values:
        0 - DEAL_TYPE_BUY - Buy
        1 - DEAL_TYPE_SELL - Sell
        2 - DEAL_TYPE_BALANCE - Balance
        3 - DEAL_TYPE_CREDIT - Credit
        4 - DEAL_TYPE_CHARGE -Additional charge
        5 - DEAL_TYPE_CORRECTION - Correction
        6 - DEAL_TYPE_BONUS - Bonus
        7 - DEAL_TYPE_COMMISSION - Additional commission
        8 - DEAL_TYPE_COMMISSION_DAILY - Daily commission
        9 - DEAL_TYPE_COMMISSION_MONTHLY - Monthly commission
        10 - DEAL_TYPE_COMMISSION_AGENT_DAILY - Daily agent commission
        11 - DEAL_TYPE_COMMISSION_AGENT_MONTHLY - Monthly agent commission
        12 - DEAL_TYPE_INTEREST - Interest rate
        13 - DEAL_TYPE_BUY_CANCELED - Canceled buy deal. There can be a situation when a previously executed buy deal is canceled. In this case, the type of the previously executed deal (DEAL_TYPE_BUY) is changed to DEAL_TYPE_BUY_CANCELED, and its profit/loss is zeroized. Previously obtained profit/loss is charged/withdrawn using a separated balance operation
        14 - DEAL_TYPE_SELL_CANCELED - Canceled sell deal. There can be a situation when a previously executed sell deal is canceled. In this case, the type of the previously executed deal (DEAL_TYPE_SELL) is changed to DEAL_TYPE_SELL_CANCELED, and its profit/loss is zeroized. Previously obtained profit/loss is charged/withdrawn using a separated balance operation
        15 - DEAL_DIVIDEND - Dividend operations
        16 - DEAL_DIVIDEND_FRANKED - Franked (non-taxable) dividend operations
        17 - DEAL_TAX - Tax charges
    """
    if x==0:
        # Compra
        return 'DEAL_TYPE_BUY'
    elif x==1:
        # Venda
        return 'DEAL_TYPE_SELL'
    elif x== 2:
        # Saldo
        return 'DEAL_TYPE_BALANCE'
    elif x== 3:
        # Crédito
        return 'DEAL_TYPE_CREDIT'
    elif x== 4:
        # Cobrança adicional
        return 'DEAL_TYPE_CHARGE'
    elif x==5 :
        # Correção
        return 'DEAL_TYPE_CORRECTION'
    elif x== 6:
        # Bonus
        return 'DEAL_TYPE_BONUS'
    elif x==7:
        # Comissão adicional
        return 'DEAL_TYPE_COMMISSION'
    elif x== 8:
        # Comissão diária
        return 'DEAL_TYPE_COMMISSION_DAILY'
    elif x== 9:
        # Comissão mensal
        return 'DEAL_TYPE_COMMISSION_MONTHLY'
    elif x== 10:
        # Comissão de agente diário
        return 'DEAL_TYPE_COMMISSION_AGENT_DAILY'
    elif x== 11:
        # Comissão de agente mensal
        return 'DEAL_TYPE_COMMISSION_AGENT_MONTHLY'
    elif x== 12:
        # Taxa de juros
        return 'DEAL_TYPE_INTEREST'
    elif x== 13:
        # Operação de compra cancelada. Pode haver uma situação quando uma operação de compra executada anteriormente é cancelada. Neste caso, o tipo de transação executada anteriormente (DEAL_TYPE_BUY) é alterada para DEAL_TYPE_BUY_CANCELED, e seu lucro/prejuízo é zerado Lucro/prejuízo obtido anteriormente é cobrado/sacado usando uma operação de saldo separada
        return 'DEAL_TYPE_BUY_CANCELED'
    elif x==14 :
        # Operação de venda cancelada. Pode haver uma situação quando uma operação de venda executada anteriormente é cancelada. Neste caso, o tipo da operação executada anteriormente (DEAL_TYPE_SELL) é alterada para DEAL_TYPE_SELL_CANCELED, e seu lucro/prejuízo é zerado. Lucro/prejuízo obtido anteriormente é cobrado/sacado usando uma operação de saldo separada
        return 'DEAL_TYPE_SELL_CANCELED'
    elif x==15 :
        # Operação de dividendos
        return 'DEAL_DIVIDEND'
    elif x== 16:
        # Operação de dividendos franqueados (não tributáveis)
        return 'DEAL_DIVIDEND_FRANKED'
    elif x== 17:
        # Cálculo do imposto
        return 'DEAL_TAX'
    else:
        return 'UNKNOWN'


def deal_reason(x):
    """
        Returns a string with the reason of a deal. Possible values:
        0 - DEAL_REASON_CLIENT - The deal was executed as a result of activation of an order placed from a desktop terminal
        1 - DEAL_REASON_MOBILE - The deal was executed as a result of activation of an order placed from a mobile application
        2 - DEAL_REASON_WEB - The deal was executed as a result of activation of an order placed from the web platform
        3 - DEAL_REASON_EXPERT - The deal was executed as a result of activation of an order placed from an MQL5 program or mt5se,  i.e. an Expert Advisor or a script
        4 - DEAL_REASON_SL  - The deal was executed as a result of Stop Loss activation
        5 - DEAL_REASON_TP - The deal was executed as a result of Take Profit activation
        6 - DEAL_REASON_SO - The deal was executed as a result of the Stop Out event
        7 - DEAL_REASON_ROLLOVER - The deal was executed due to a rollover
        8 - DEAL_REASON_VMARGIN - The deal was executed after charging the variation margin
        9 - DEAL_REASON_SPLIT - The deal was executed after the split (price reduction) of an instrument, which had an open position during split announcement
    """
    if x==0:
        #Transação realizada como resultado da ativação de uma ordem colocada a partir de um terminal desktop
        return 'DEAL_REASON_CLIENT'
    elif x==1:
        #Transação realizada como resultado da ativação de uma ordem colocada a partir de um aplicativo móvel
        return 'DEAL_REASON_MOBILE'
    elif x==2:
        #Transação realizada como resultado da ativação de uma ordem colocada a partir da plataforma web
        return 'DEAL_REASON_WEB'
    elif x== 3:
        #Transação realizada como resultado da ativação de uma ordem colocada a partir de um programa MQL5, Expert Advisor ou script
        return 'DEAL_REASON_EXPERT'
    elif x== 4:
        #Transação realizada como resultado da ativação de uma ordem Stop Loss
        return 'DEAL_REASON_SL'
    elif x== 5:
        #Transação realizada como resultado da ativação de uma ordem Take Profit
        return 'DEAL_REASON_TP'
    elif x== 6:
        #Transação realizada como resultado do evento Stop Out
        return 'DEAL_REASON_SO'
    elif x== 7:
        #Transação realizada devido à transferência da posição
        return 'DEAL_REASON_ROLLOVER'
    elif x== 8:
        #Transação realizada após creditada/debitada a margem de variação
        return 'DEAL_REASON_VMARGIN'
    elif x== 9:
        #Transação realizada após o fracionamento (redução do preço) do instrumento que tinha a posição aberta durante o fracionamento    
        return 'DEAL_REASON_SPLIT'
    else:
        return 'UNKNOWN'




"""
Returns intraday (1 minute) bars.
     It is the same as se.get_bars(symbol,day,timeFrame=se.INTRADAY)
    see get_bars()
def get_intraday_bars(symbol, day):
    return get_bars(symbol,day,timeFrame=INTRADAY)
"""

"""
"""

def roll_bars(bars, new_bars):
    len_new=len(new_bars)
    if len(bars)<=len_new:
        print('The length of bars',len(bars),' should be bigger than the lenght of new_bars ',len(new_bars))
        return bars
    for i in range(len_new):
        bars=bars.drop(i)
    bars=bars.append(new_bars)
    bars=bars.reset_index()
    del bars['index']
    return bars




def get_close_prices(assets,start,end=None,timeFrame=DAILY):
    """
 Returns a pandas.DataFrame like the one below for a group of assets. It is similar to get_multi_bars.
                XOM        RRC        BBY         MA        PFE        JPM
date
2010-01-04  54.068794  51.300568  32.524055  22.062426  13.940202  35.175220
2010-01-05  54.279907  51.993038  33.349487  21.997149  13.741367  35.856571
2010-01-06  54.749043  51.690697  33.090542  22.081820  13.697187  36.053574
..
# Note that 'date' column is the index, the others are assets' close prices
"""
    if not connected:
        print("In order to use this function, you must be connected to the Stock Exchange. Use function connect()")
        return
    df=pd.DataFrame()
    first=True
    for asset in assets:
        bars=get_bars(asset,start,end,timeFrame)
        if first:
            df['date']=bars['time']
            first=False
        df[asset]=bars['close']
    df=df.set_index('date')
    return df

def get_close_prices_from_dbars(assets,dbars,option=1):
    """
Returns a pd.DataFrame like the one below for a group of assets from given
 multi asset bars
   The price can be the close price (default) or the average of open,close,high,low
                XOM        RRC        BBY         MA        PFE        JPM
date
2010-01-04  54.068794  51.300568  32.524055  22.062426  13.940202  35.175220
2010-01-05  54.279907  51.993038  33.349487  21.997149  13.741367  35.856571
2010-01-06  54.749043  51.690697  33.090542  22.081820  13.697187  36.053574
2010-01-07  54.577045  51.593170  33.616547  21.937523  13.645634  36.767757
2010-01-08  54.358093  52.597733  32.297466  21.945297  13.756095  36.677460
 Note that 'date' column is the index, the others are assets' close prices
"""
    df=pd.DataFrame()
    first=True
    for asset in assets:
        bars=dbars[asset]
        if first:
            df['date']=bars['time']
            first=False
        if option==1:
            df[asset]=bars['close']
        elif option==2:
            df[asset]=bars['open']/4+bars['close']/4+bars['high']/4+bars['low']/4
    df=df.set_index('date')
    return df


def mean_historical_return(df,geometric=True):
    """
    Returns the historical mean, it can be geometric (default) or arithmetic
    If the data is daily, the mean is daily.
"""
    expected_returns=dict()
    assets=df.keys()
    for asset in assets:
        size=len(df[asset])
        ret=0
        if not geometric: #arithmetic
            for i in range(size-1):
                ret=ret+(df[asset][i+1]/df[asset][i]-1)
            expected_returns[asset]=ret/(size-1)
        else:
            ret=(df[asset][size-1]/df[asset][0])
            expected_returns[asset]=ret**(1/(size-1))-1         
    return expected_returns



def get_last_prices(assets,dbars=None):
    """
    Returns a dictionary with the last prices of set of assets
        the symbol tickets are the dictionary keys
"""
    last_prices=dict()
    if dbars is not None:
        for asset in assets:
            last_prices[asset]=get_last(dbars[asset])
    else:
        for asset in assets:
            bars=get_bars(asset,1,timeFrame=INTRADAY)
            last_prices[asset]=get_last(bars)
    return last_prices


def get_volume_steps(assets):
    """
    Returns a dictionary with the volume steps of set of assets
"""
    steps=dict()
    for asset in assets:
        steps[asset]=get_volume_step(asset)
    return steps


def get_curr_shares(assets=None):
    """
    Returns a dictionary with the current number of shares for a list of assets.
    If no list is provided (None), it returns a dictionary for all assets with non zero position
"""
    if assets is None:
        cshares=dict()
        pos=mt5.positions_get()
        for p in pos:
            cshares[p.symbol]=p.volume
        return cshares
    cshares=dict()
    for asset in assets:
        cshares[asset]=get_shares(asset)
    return cshares


def orders_from_weights(weights,last_prices,capital):
    """
    Returns a list of orders to adopt a portfolio defined by a set of weights (dictionary),
        a set of last prices (dictionary) and the amount of available capital 
"""
    # sort weights from highest to lowest
    weights=dict(sorted(weights.items(), key=lambda item:item[1],reverse=True))
    #round 1 - buy while never exceeds the desired weight
    assets=weights.keys()
    sum=0
    curr=dict() # current weights
    volumes=dict() #  order's volumes
    steps=get_volume_steps(assets)
    for asset in assets:
        aval_capital=weights[asset]*capital
        shares=get_affor_shares(asset,last_prices[asset],aval_capital,steps[asset])
        if shares<=0:
            curr[asset]=0
            volumes[asset]=0
        else:
            curr[asset]=(shares*last_prices[asset])/capital
            volumes[asset]=shares
        sum=sum+curr[asset]
    #round 2 - if there is remainig capital, buy more lot according weight order 
    remain_capital=capital-sum*capital
    if remain_capital<=0:
        return volumes
    for asset in weights.keys():
        s=steps[asset]
        p=last_prices[asset]
        missing=(weights[asset]-curr[asset])*capital
        while s*p<remain_capital and s*p<missing:
            s=s+steps[asset]
        if s*p>remain_capital :
            s=s-steps[asset]
        curr[asset]=curr[asset]+(s*p)/capital
        volumes[asset]=volumes[asset]+s
        remain_capital=remain_capital-s*p
        if remain_capital<=0:
            break
    return volumes


def get_new_orders_from_curr_shares(orders,curr_shares):
    """
    Get new adjusted orders considering current shares of a set of assets, and orders to buy a certain
    numbers of shares for each asset. For instance,
        if orders define buy 1000 of X and 500 of Y and currently the account has 100 shares of X and 900 shares of Y
        the new orders would be buy 900 shares of X and sell 400 shares of Y.
"""
    new_orders=dict()
    for k in orders.keys():
        new_orders[k]=orders[k]-curr_shares[k]
    return new_orders




def date(year,month,day,hour=0,min=0,sec=0):
    """
    Returns a datetime with the specified year,month,day,hour=0,min=0,sec=0
"""
    return datetime(year,month,day,hour,min,sec)


###############################
# Classes used in inverse control trader and backtest!!
class Trader:
    """
        Basic Trader class for mt5se
    """
    def __init__(self):
        """
            Trader's constructor
        """
        pass
    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
    #   and setups the operation
    def setup(self,dbars):
        """
            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
       and prepares the trader for execution. It is called once, just after trader creation
        """
        pass

    def trade(self,dbars):
        """
            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
       and returns a list of orders with up to one order for each asset. It is called every trading cycle
        """
        pass

    def orders_result(self, exec_orders):
        """
            Receive a list of executed orders. It is called after Trader.trade() with the list of orders given by Trader.trader that were really executed.
            Receives orders_res_list - a list of the orders really executed with real volume and price
       Each result is a dictionary 'order' with:
        order['symbol'] - string
        order['isSellOrder'] - boolean
        order['shares'] - float
        order['price'] - float
        It is called once at every trading cycle
     """
        pass

    def ending(self,dbars):
        """
            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
        It is called once, just before trader's end of execution
        """
        pass
 

class Analyst:
    """
        Basic Analyst class for mt5se. It may be used for Trader to get target prices.
    """
    def __init__(self):
        """
            Analyst's constructor
        """
        pass
    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
    #   and setups the operation
    def setup(self,dbars):
        """
            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
       and prepares the analyst for execution. It is called once, just after trader creation
        """
        pass

    def analyze(self,dbars):
        """
            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
       and returns a list of target price for each asset (-1 if not available for a given asset). It is called every trading cycle
        """
        pass
   
    def ending(self,dbars):
        """
            Receives dbars[asset] - a bars dataframe for each asset in a dictionary
        It is called once, just before analyst's end of execution
        """
        pass


# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17

##########################################



import MetaTrader5 as mt5
import pandas as pd 
import numpy as np 

import random
from datetime import datetime
from datetime import timedelta
# importamos o módulo pytz para trabalhar com o fuso horário
import pytz
from pytz import timezone


############
from mt5se.mt5se import *
import mt5se.tech as tech
import mt5se.finmath as finmath
import mt5se.sampleTraders as sampleTraders
import mt5se.analysts as analysts
import mt5se.backtest as backtest
import mt5se.operations as operations
import mt5se.ai_utils as ai_utils




# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17

import pandas as pd 
import numpy as np 
import mt5se.mt5se as se
from scipy import stats



def rsi(returns):
    """
    	Returns the RSI (Relative Strengh Index) of a given serie of returns.
            if the parameter is a pandas.DataFrame it uses the function mt5se.get_return() to get the
            serie of returns
    """
    if type(returns)==pd.core.frame.DataFrame:
        returns=se.get_returns(returns)
    u=0.0
    uc=0
    d=0.0
    dc=0
    for r in returns:
        if r>=0:
            u=u+r
            uc=uc+1
        else:
            d=d+abs(r)
            dc=dc+1
    if uc>0:
        u=u/uc
    if dc>0:
        d=d/dc   
    if d==0:
        d=1.0
    ifr=100*( 1 - 1/(1+u/d))
    return ifr


def slope(serie):
    """
    	Returns the angular coefficient of linear regression (slope)
          for a serie of prices in regular intervals
    """
    x=np.array(range(len(serie)))
    #y=np.array(serie)
    s=stats.linregress(x,serie)
    return s.slope

# equals to slope
def trend(serie):
    """
      	Returns the angular coefficient of linear regression (slope)
          for a serie of prices in regular intervals 
          Same as function slope()
    """
    return slope(serie)

def ma(serie,length=10):
    """"
        Returns the moving average of lenght points.
            In the fist points (0-length), it calculcates the average from 0 to index.
            So, the first ma is equal to the first number of the serie, the second is the average between the first and second numbers of the serie, and so on
    """
    mov_avg=[]
    for i in range(len(serie)):
        if i <=length:
            mov_avg.append(np.mean(serie[0:i+1]))
        else:
            mov_avg.append(np.mean(serie[i-9:i+1]))
    return mov_avg

    


import mt5se as se
import pandas as pd
import numpy as np


## Defines the RandomForestAnalyst
from sklearn.ensemble import RandomForestClassifier
from sklearn.preprocessing import KBinsDiscretizer


class RsiAnalyst(se.Analyst):
    def setup(self,dbars):
        assets=list(dbars.keys())
        df=se.get_close_prices_from_dbars(assets,dbars)
        # train model
        self.mu = se.mean_historical_return(df)
        self.alpha=0.5
        self.dbars=dbars


    def analyze(self,dbars):
        assets=dbars.keys()
        returns=dict()
        alpha=self.alpha
        for asset in assets:
            bars=dbars[asset]
            # number of shares that you can buy of asset 
            rsi=se.tech.rsi(bars)
            er=self.mu[asset]
            if rsi>=70: 
                exp_ret=er+alpha*abs(er)          
            elif  rsi<70:
                exp_ret=er-alpha*abs(er)                  
            returns[asset]=exp_ret
        return returns    

class MAAnalyst(se.Analyst):
    def setup(self,dbars):
        assets=list(dbars.keys())
        df=se.get_close_prices_from_dbars(assets,dbars)
        # train model
        mu = se.mean_historical_return(df)
        self.alpha=0.5
        self.mu=mu
        self.period=10

    def analyze(self,dbars):
        assets=dbars.keys()
        returns=dict()
        for asset in assets:
            bars=dbars[asset]
            # number of shares that you can buy of asset 
            er=self.mu[asset]
            m=np.mean(bars['close'][-self.period:])
            if se.tech.trend(bars['close'])>0 and bars['close'].iloc[-1]<m:
                exp_ret=er+self.alpha*abs(er)
            elif se.tech.trend(bars['close'])<0 and m<bars['close'].iloc[-1]:
                exp_ret=er-self.alpha*abs(er)
            else:
                exp_ret=None
            returns[asset]=exp_ret
        return returns  


class MACDAnalyst(se.Analyst):
    def setup(self,dbars):
        self.period=26
        self.short_period=12
        self.signal=9
        assets=list(dbars.keys())
        df=se.get_close_prices_from_dbars(assets,dbars)
        mu = se.mean_historical_return(df)
        self.alpha=0.5
        self.dbars=dbars
        self.mu=mu
        self.lastMacdUnderSignal=True
        if len(df)<self.period+self.signal:
            print('The setup period (prestart-start) should have at least ',self.period+self.signal,' data points')
            return
        # train model


    def analyze(self,dbars): #recebe um numero de barras igual a prestart-start porem ja roladas
        assets=dbars.keys()
        returns=dict()
        MacdUnderSignal=True
        lastMacdUnderSignal=True
        alpha=self.alpha
        for asset in assets:
            #self.dbars[asset]=se.roll_bars(self.dbars[asset],ndbars[asset])
            bars=dbars[asset]
            # number of shares that you can buy of asset 
            #m=np.mean(bars['close'])
            longer=np.mean(bars['close'][-self.period:])
            shorter=np.mean(bars['close'][-self.short_period:])
            signaling=np.mean(bars['close'][-self.signal:])
            er=self.mu[asset]
            if((shorter-longer)>signaling):
                MacdUnderSignal=False
            else:
                MacdUnderSignal=True
            if(not MacdUnderSignal and  lastMacdUnderSignal): # alta: compra
    	        exp_ret=er+alpha*abs(er)
    	        #print('buy ',asset,' expr_r=', exp_ret)
            elif(MacdUnderSignal and  not lastMacdUnderSignal): #baixa: vende
                exp_ret=er-alpha*abs(er)
                #print('sell ',asset,' expr_r=',exp_ret)
            else: # indefinido faz nada
                exp_ret=None
                #print('Nothing ',asset,' expr_r=',exp_ret)  
            lastMacdUnderSignal= MacdUnderSignal  	        
            returns[asset]=exp_ret
        return returns  


class RandomForestAnalyst(se.Analyst):
    def setup(self,dbars):
        assets=list(dbars.keys())
        df=se.get_close_prices_from_dbars(assets,dbars)
        mu = se.mean_historical_return(df)
        self.clf=dict()
        for asset in assets:
            bars=dbars[assets[0]]
            bars=bars.copy()
            # remove irrelevant info
            if 'time' in bars:
                del bars['time']
            timeFrame=10 # it takes into account the last 10 bars
            horizon=1 # it project the closing price for next bar
            target='close' # name of the target column
            ds=se.ai_utils.bars2Dataset(bars,target,timeFrame,horizon)
            X=se.ai_utils.fromDs2NpArrayAllBut(ds,['target'])
            discretizer = KBinsDiscretizer(n_bins=3, encode='ordinal', strategy='uniform') 
            # creates the discrete target
            #print('ds=',ds)
            ds['target']=se.ai_utils.discTarget(discretizer,ds['target'])
            Y=se.ai_utils.fromDs2NpArray(ds,['target'])
            # train model for each asset
            clf = RandomForestClassifier(n_estimators=10)
            clf = clf.fit(X, Y)
            self.clf[asset]=clf
        self.steps=dict()
        self.alpha=0.5
        self.dbars=dbars
        self.mu=mu
    def analyze(self,dbars):
            assets=dbars.keys()
            returns=dict()
            timeFrame=10 # it takes into account the last 10 bars
            horizon=1 # it project the closing price for next bar
            target='close' # name of the target column
            for asset in assets:
                #print('Tempo1.1=',datetime.now())
                # get new information (bars), transform it in X
                bars=dbars[asset]
                bars=bars.copy()
                #remove irrelevant info
                if 'time' in bars:
                    del bars['time']
                # convert from bars to dataset
                ds=se.ai_utils.bars2Dataset(bars,target,timeFrame,horizon)
                # Get X fields
                X=se.ai_utils.fromDs2NpArrayAllBut(ds,['target'])
                # predict the result, using the latest info
                p=self.clf[asset].predict([X[-1]])
                er=self.mu[asset]
                if p==2:
                    exp_ret=er+self.alpha*abs(er)      #buy it
                    #print("buy ",asset)
                elif p==0:
                    #sell it
                    exp_ret=er-self.alpha*abs(er)
                    #print("sell ",asset)
                else:
                    exp_ret=None
                    #print("do nothing ",asset)
                returns[asset]=exp_ret
            return returns    


def ensembleAnalyses(analysts_mus,mu):
    expected_returns=dict()
    assets=analysts_mus[0].keys()
    for asset in assets:
        count=0
        expected_returns[asset]=0
        for i in range(len(analysts_mus)): # gets expected return of each analyst
            if analysts_mus[i][asset]!=None:
                expected_returns[asset]=expected_returns[asset]+analysts_mus[i][asset]
                count=count+1
        if count>0:
            expected_returns[asset]=expected_returns[asset]/count
        else:
            expected_returns[asset]=0 #mu[asset] # if no analyst informed anything uses zero instead of the historical value
    return pd.Series(expected_returns)



"""
  Analyst that ensembles the analyses of several analyst in order to form just one analysis of several assets
    It uses a weigthed mean to ensemble the analyses. If weights are not provided, then it uses a simple arithmetic mean
"""
class EnsembleAnalyst(se.Analyst):
    def __init__(self):
        self.analysts=dict()
        self.weights=dict()

    def add_analyst(self,analyst,name,weight=1):
        self.analysts[name]=analyst
        self.weights[name]=weight  # weight of each analyst, if not provided it will be uniform!

    def setup(self,dbars):
        s=0
        for name in self.analysts.keys():
            self.analysts[name].setup(dbars)
            s=s+self.weights[name]
        for name in self.analysts.keys():
            self.weights[name]=self.weights[name]/s   # defines the relative weight for each analyst 
        
    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
    #   and returns a dictionary with the target price for each asset
    def analyze(self,dbars):
        analysts_prices=dict()
        for name in self.analysts.keys():
            prices=self.analysts[name].analyze(dbars)
            analysts_prices[name]=prices
        assets=dbars.keys()
        return self.ensembleExpectReturns(analysts_prices,assets)

    # Receives dbars[asset] - a bars dataframe for each asset in a dictionary
    #   and frees resources used by the Analyst
    def ending(self,dbars):
        for name in self.analysts.keys():
            self.analysts[name].setup(dbars)

    # analyst_prices is a dict with entry for each analyst, and for each analyst it is a dict for 
    # the target price for each asset
    def ensembleExpectReturns(self,analysts_prices,assets):
        expectReturns=dict()
        for asset in assets:
            w_sum=0
            expectReturns[asset]=0
            for anl in analysts_prices.keys(): # list of analysts
                if analysts_prices[anl][asset]!=None and analysts_prices[anl][asset]>0: # if analyst has estimate, it is included!
                    expectReturns[asset]=expectReturns[asset]+analysts_prices[anl][asset]*self.weights[anl]
                    #print('anl=',anl,' w=',self.weights[anl], ' anl price',analysts_prices[anl][asset],)
                    w_sum=w_sum+self.weights[anl]
            if w_sum>0:
                expectReturns[asset]=expectReturns[asset]/w_sum # calculates mean target price of all that did not abstained
            else:
                expectReturns[asset]=-1 # if no analyst informed anything it abstain from giving a target price
        return expectReturns



# This file is part of the mt5se package
#  mt5se home: https://github.com/paulo-al-castro/mt5se
# Author: Paulo Al Castro
# Date: 2020-11-17


## chamadas que sao enviadas para a corretora (brokerage), logo exigem comunicação com a mesma 


# broker module
import MetaTrader5 as mt5
import pandas as pd 
import numpy as np 

import random
from math import *
from datetime import datetime
from datetime import timedelta
# importamos o módulo pytz para trabalhar com o fuso horário
import pytz
from pytz import timezone



sptz=pytz.timezone('Brazil/East')
etctz=pytz.timezone('etc/utc') # os tempos sao armazenados na timezone ETC (Greenwich sem horario de verao)

path=None # Metatrader program file path
datapath=None # Metatrader path to data folder
commonDatapath=None # Metatrader common data path
company=None  #broker name
platform=None  # digital plataform (M)


"""
   login,                    // número da conta  (TODO)
   password="PASSWORD",      // senha
   server="SERVER",          // nome do servidor como definido no terminal
   timeout=TIMEOUT           // tempo de espera esgotado
"""
def connect( ):
	#if not se.connect():
	#print(“Error on connection”, se.last_error())
	#exit():
    res= mt5.initialize()
    if not res:
        ac=mt5.terminal_info()
        path=ac.path
        datapath=ac.data_path
        commonDatapath=ac.commondata_path
        company=x.company
        platform=x.name
    return res


def accountInfo():
#acc=se.accountInfo()    # it returns a dictionary
#acc['login']   # Account id
#acc['balance'] # Account balance in the deposit currency
# acc['equity'] # Account equity in the deposit currency
#acc['margin']  #Account margin used in the deposit currency
#acc['margin_free'] # Free margin of an account in the deposit currency
#acc['assets'] # The current assets of an account
# acc['name'] #Client name
#  acc['server'] # Trade server name
#  acc['currency'] # Account currency, BRL for Brazilian Real 
    account_info = mt5.account_info()
    #print("account info")
    return account_info
"""
    returns the current number of assets of the given symbol
"""
def getShares(symbolId):
   pos= mt5.positions_get(symbol=symbolId)
   if pos!=None and pos!=():
      d=pos[0]._asdict() 
      return d['volume']
   else:
       return 0

   return pos['volume']


"""
  It returns if the market is open or not for new orders.
     Note that markets can close in different times for different assets, therefore
     you need to inform the target asset. The default target assets is se stock.
     It there is no tick for 60 seconds, the market is considered closed!
"""
def isMarketOpen(asset='seSA3'):
   # si=mt5.symbol_info(asset)
   # if si!=None:
  #      if si.trade_mode==mt5.SYMBOL_TRADE_MODE_FULL: # it does not work in XP/se (always True)
  #          return True
  #      else:
  #          return False
  #  return False
  t_secs=mt5.symbol_info_tick(asset).time # time in seconds
  now_dt=datetime.now(etctz)+timedelta(hours=-3)
  last_tick_dt=datetime.fromtimestamp(t_secs,etctz)
  #print(last_tick_dt)
  #print(now_dt)
  if now_dt>last_tick_dt+timedelta(seconds=60):
      return False
  else: 
      return True

"""
  It returns if the market is still open but just for closing orders.
     Note that markets can close in different times for different assets, therefore
     you need to inform the target asset. The default target assets is se stock
"""
#def isMarketClosing(asset='seSA3'): # it does not work in XP/se (always false)
#    si=mt5.symbol_info(asset)
#    if si!=None:
 #       if si.trade_mode==mt5.SYMBOL_TRADE_MODE_CLOSEONLY:
  #          return True
  #      else:
  #          return False
  #  return False


   
"""
    returns the max volume of shares thay you can buy, with your free margin
        it also observes the volume step (a.k.a minimum number of shares you can trade)
"""
def getAfforShares(assetId,money=None,price=None):
    if money==None:
        money=mt5.account_info().margin_free
    if price==None:
        close=mt5.symbol_info_tick(assetId).last
    else:
        close=price
    step=mt5.symbol_info(assetId).volume_step
    free=0
    while free*close<money:
        free=free+step
    return free-step

def getSharesStep(assetId,money=None):
    return mt5.symbol_info(assetId).volume_step
 

def sendOrder(order):
    if order==None:
        return False
    # enviamos a solicitação de negociação
    result = mt5.order_send(order)
    if result.retcode != mt5.TRADE_RETCODE_DONE:  # if error
        print("Sent order failed < {} > retcode={}".format(result.comment,result.retcode))
        # solicitamos o resultado na forma de dicionário e exibimos elemento por elemento
        dic=result._asdict()
        setLastError(dic['comment'])
       # for field in dic.keys():
       #     print("   {}={}".format(field,dic[field]))
       #     #se esta for uma estrutura de uma solicitação de negociação, também a exibiremos elemento a elemento
       #     if field=="request":
       #         traderequest_dict=dic[field]._asdict()
       #         for tradereq_filed in traderequest_dict:
       #             print("       traderequest: {}={}".format(tradereq_filed,traderequest_dict[tradereq_filed]))
        return False
    else:
        return True

    
def cancelOrder(o):# TO DO
   # action= TRADE_ACTION_REMOVE
    print("To do....")

def numOrders(): #returns the number of active orders
    result=mt5.orders_total()
    if result==None:
        setLastError("Error on getting orders total")
        return -1
    else:
        return result

#order fields  description:
    #order_id | buy_sell | volume | price | sl | tp | 
    #ticket | time_setup  time_setup_msc  time_expiration  type  type_time  type_filling  state  magic  
    # volume_current  price_open   sl   tp  price_current  symbol comment external_id
    #   ulong                         magic;            // Expert Advisor -conselheiro- ID (número mágico)
   #  ulong                         order;            // Bilhetagem da ordem
   #string                        symbol;           // Símbolo de negociação
  # double                        volume;           // Volume solicitado para uma encomenda em lotes
  # double                        price;            // Preço
  # double                        stoplimit;        // Nível StopLimit da ordem
  # double                        sl;               // Nível Stop Loss da ordem
  # double                        tp;               // Nível Take Profit da ordem
  # ulong                         deviation;        // Máximo desvio possível a partir do preço requisitado
 #  ENUM_ORDER_TYPE               type;             // Tipo de ordem
    #  ORDER_TYPE_BUY  Ordem de Comprar a Mercado
    #  ORDER_TYPE_SELL Ordem de Vender a Mercado
    #  ORDER_TYPE_BUY_LIMIT Ordem pendente Buy Limit
    #  ORDER_TYPE_SELL_LIMIT Ordem pendente Sell Limit
    #  ORDER_TYPE_BUY_STOP Ordem pendente Buy Stop
    #  ORDER_TYPE_SELL_STOP Ordem pendente Sell Stop
    #  ORDER_TYPE_BUY_STOP_LIMIT Ao alcançar o preço da ordem, uma ordem pendente Buy Limit é colocada no preço StopLimit
    #  ORDER_TYPE_SELL_STOP_LIMIT Ao alcançar o preço da ordem, uma ordem pendente Sell Limit é colocada no preço StopLimit
    #  ORDER_TYPE_CLOSE_BY  Ordem de fechamento da posição oposta
  # ENUM_ORDER_TYPE_FILLING       type_filling;     // Tipo de execução da ordem
    #ORDER_FILLING_FOK  Esta política de preenchimento significa que uma ordem pode ser preenchida somente na quantidade especificada. Se a quantidade desejada do ativo não está disponível no mercado, a ordem não será executada. 
  #  ENUM_ORDER_TYPE_TIME          type_time;        // Tipo de expiração da ordem
    # ORDER_TIME_DAY     Ordem válida até o final do dia corrente de negociação
  # datetime                      expiration;       // Hora de expiração da ordem (para ordens do tipo ORDER_TIME_SPECIFIED))
  # string                        comment;          // Comentário sobre a ordem
  # ulong                         position;         // Bilhete da posição
  # ulong                         position_by;      // Bilhete para uma posição oposta

def getOrders():  # returns a dataframe with all active orders
    orders=mt5.orders_get()
    if orders == None or len(orders)==0:
        print("No orders, error code={}".format(mt5.last_error()))
        return None
    else:
        print("Total orders:",len(orders))
        df=pd.DataFrame(list(orders),columns=orders[0]._asdict().keys())
        return df
      


def getDailYBars(symbol, start,end=None): # sao inclusas barras com  tempo de abertura <= end.
    # definimos o fuso horário como UTC
    #timezone = pytz.timezone("Etc/UTC")
    if end==None:
        end=datetime.now()
    if type(start).__name__!='datetime':
        if type(start).__name__!='int':
            print('Error, start should be a datetime from package datetime or int')
        else:
            start_day=datetime.now() #- timedelta(days=start)
            rates=mt5.copy_rates_from(symbol,mt5.TIMEFRAME_D1,start_day,start)
             # criamos a partir dos dados obtidos DataFrame
            rates_frame=pd.DataFrame(rates)
            rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
            return rates_frame
    else:
       rates=mt5.copy_rates_range(symbol,mt5.TIMEFRAME_D1,start,end)
       # criamos a partir dos dados obtidos DataFrame
       rates_frame=pd.DataFrame(rates)
       rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
       return rates_frame



    return x

DAILY=mt5se.mt5se.DAILY

def get_bars(symbol, start,end=None,timeFrame=DAILY):
 # definimos o fuso horário como UTC
    #timezone = pytz.timezone("Etc/UTC")
    if symbol==None or type(symbol)!=str:
        return None
    else:
        symbol=symbol.upper()
    if timeFrame==DAILY:
        timeFrame=mt5.TIMEFRAME_D1
    elif timeFrame==INTRADAY:
        timeFrame=mt5.TIMEFRAME_M1
    else:
        timeFrame=mt5.TIMEFRAME_D1
    if end==None:
        end=datetime.now()
    if type(start).__name__!='datetime':
        if type(start).__name__!='int':
            print('Error, start should be a datetime from package datetime or int')
        else:
            start_day=datetime.now() #- timedelta(days=start)
            rates=mt5.copy_rates_from(symbol,timeFrame,start_day,start)
             # criamos a partir dos dados obtidos DataFrame
            rates_frame=pd.DataFrame(rates)
            if len(rates_frame)>0:
                rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
            return rates_frame
    else:
       rates=mt5.copy_rates_range(symbol,timeFrame,start,end)
       # criamos a partir dos dados obtidos DataFrame
       rates_frame=pd.DataFrame(rates)
       rates_frame['time']=pd.to_datetime(rates_frame['time'], unit='s')
       return rates_frame


def getIntradayBars(symbol, day):
    # definimos o fuso horário como UTC
    #timezone = pytz.timezone("Etc/UTC")
    if type(day).__name__!='datetime':
        print('Error, start should be a datetime from package datetime')
    else:
       rates=mt5.copy_rates_range(symbol,mt5.TIMEFRAME_M1,\
       day,datetime(day.year,day.month,day.day,23,59))
       # criamos a partir dos dados obtidos DataFrame
       return pd.DataFrame(rates)
##############
# datetime auxiliary functions



 


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mt5se",
    version="0.2.39",
    author="Paulo Andre Lima de Castro",
    license='MIT', 
    author_email="paulo.al.castro@gmail.com",
    description="mt5se provides access to Stock Exchanges to python programs through Metatrader and some brokers. It can be used with NYSE,Nasdaq, B3(Brazilian Stock Exchange)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/paulo-al-castro/mt5se/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)